package br.edu.fei.dcc.raciocinioespacial.editor.commands;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Properties;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchWindow;

import br.edu.fei.dcc.raciocinioespacial.editor.application.Application;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.ShapesEditor;
import br.edu.fei.dcc.raciocinioespacial.simulador.execucao.Simulador;

public class RunSimAction extends Action {
	
	IWorkbenchWindow workbenchWindow;
	
	public RunSimAction(IWorkbenchWindow window) {
		super.setId("executeSimulation");
		super.setText("Execute Simulation");
		super.setImageDescriptor(ImageDescriptor.createFromFile(Application.class, "icons/execute.png"));
		this.workbenchWindow = window;
	}
	
	public void run() {
		if(workbenchWindow.getActivePage().getActiveEditor()!=null)
		{
			ShapesEditor editor = (ShapesEditor)workbenchWindow.getActivePage().getActiveEditor();
			Transformacao modelo = new Transformacao(editor.getModel());
			if (modelo.getSimulacao().getAgent() != null)
			{
				IPath p = new Path(getClass().getProtectionDomain().getCodeSource().getLocation().getPath());
		        p = p.removeTrailingSeparator();
		        p = p.removeLastSegments(1).makeAbsolute();
		        p = p.makeAbsolute();
		        
		        trataPrologPath(p.toString());
		        
				Simulador app = new Simulador(modelo.getSimulacao(), p.toString());
				app.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				app.setVisible(true);
			}
			else
				MessageDialog.openWarning(workbenchWindow.getShell(), "What about the agent?", "The simulation must have agent.");
		}else
		{
			MessageDialog.openWarning(workbenchWindow.getShell(), "Error on executing the simulation", "There is no simulation to execute.");
		}
	}

	private void trataPrologPath(String fullPath) {
		try {
			File fileProp = new File(fullPath+"\\Properties\\filespath.properties");
			String sPrologPath = null;
			Properties prop = new Properties();
			
			if (fileProp.exists()) {
				prop.load(new FileReader(fileProp));
				sPrologPath = prop.getProperty("prologexecpath");
			}
			
			if (sPrologPath == null || sPrologPath.length() == 0)
			{
				sPrologPath = JOptionPane.showInputDialog("The Prolog path cannot be found."
						+ "Please, type the full Prolog path, including the file name.",
				"C:\\Prolog\\bin\\plwin.exe");
				
				prop.setProperty("prologexecpath", sPrologPath);
				prop.store(new FileWriter(fileProp), "");
			}
			
		} catch (Exception ex) {
			// TO NEM AE!
		}
	}
}
