package br.edu.fei.dcc.raciocinioespacial.editor.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Agent2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Pedestrian2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Ponto2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Street2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Vehicle2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Agent2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.IEntidadeMovel;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Pedestrian2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Street2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Vehicle2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.simulacao.Simulacao;

/**
 * Class that transform a simulation with our objects on a ShapesDiagram
 * to be used in GEF Editor or even the inverse
 */
public class Transformacao {
	private ShapesDiagram diagram;
	private Simulacao simulacao;
	
	public Transformacao(ShapesDiagram diagram)
	{
		this.diagram = diagram;
		this.transformToSimulacao();
	}

	public Transformacao(Simulacao simulacao)
	{
		this(simulacao, new ShapesDiagram());
	}

	public Transformacao(Simulacao simulacao, ShapesDiagram diagram) {
		this.simulacao = simulacao;
		this.diagram = diagram;
		this.transformToShapesDiagram();
	}

	private void transformToSimulacao() {
		if(this.diagram!=null)
		{
			this.simulacao = new Simulacao(this.diagram.getNome(), this.diagram.getIntervalo(),
					this.diagram.getPerfilProfundidade(), this.diagram.getIsShowDlgDetalhes(), this.diagram.getIsPaintRoads());
			
			for(Shape x : diagram.getChildren())
			{
				if(x instanceof Agent2DShape)
				{
					this.simulacao.setAgent(this.transformAgent2DShapeToAgent2D((Agent2DShape)x));
				}
				else if(x instanceof Vehicle2DShape)
				{
					this.simulacao.getObjetos().add(this.transformVehicle2DShapeToVehicle2D((Vehicle2DShape)x));
				}
				else if(x instanceof Pedestrian2DShape)
				{
					this.simulacao.getObjetos().add(this.transformPedestrian2DShapeToPedestrian2D((Pedestrian2DShape)x));
				}
				else if(x instanceof Street2DShape)
				{
					this.simulacao.getStreetsArray().add(this.transformStreet2DShapeToStreet2D((Street2DShape)x));
				}
			}
		}
	}

	private IEntidadeMovel transformPedestrian2DShapeToPedestrian2D(Pedestrian2DShape obj) {
		if(obj!=null)
		{
			Pedestrian2D retorno = new Pedestrian2D(obj.getLocalization().x, obj.getLocalization().y, obj.getSize().width, obj.getSize().height, obj.getName());
			retorno.setVelocidade(obj.getVelocity());
			for(Ponto2DShape p : obj.getPoints())
			{
				retorno.getPontos().add(new java.awt.Point(p.getLocalization().x, p.getLocalization().y));
			}
			return retorno;
		}
		return null;
	}
	
	private Street2D transformStreet2DShapeToStreet2D(Street2DShape obj)
	{
		if(obj!=null)
		{
			Street2D retorno = new Street2D(obj.getLocalization().x, obj.getLocalization().y, 
					obj.getSize().height,obj.getSize().width, obj.getName());
			return retorno;
		}
		return null;
	}
	
	private IEntidadeMovel transformVehicle2DShapeToVehicle2D(Vehicle2DShape obj) {
		if(obj!=null)
		{
			Vehicle2D retorno = new Vehicle2D(obj.getLocalization().x, obj.getLocalization().y, obj.getSize().width, 
					obj.getSize().height, obj.getName());
			retorno.setVelocidade(obj.getVelocity());
			retorno.setSentido(obj.getSentido());
			for(Ponto2DShape p : obj.getPoints())
			{
				retorno.getPontos().add(new java.awt.Point(p.getLocalization().x, p.getLocalization().y));
			}
			return retorno;
		}
		return null;
	}

	private Agent2D transformAgent2DShapeToAgent2D(Agent2DShape obs) {
		if(obs!=null)
		{
			Agent2D retorno = new Agent2D(obs.getLocalization().x, obs.getLocalization().y, obs.getSize().width, 
					obs.getSize().height, obs.getName(), obs.getProfundidade(), obs.getAngulo());
			
			retorno.setVelocidade(obs.getVelocity());
			retorno.setSentido(obs.getSentido());
			
			for(Ponto2DShape p : obs.getPoints())
			{
				retorno.getPontos().add(new java.awt.Point(p.getLocalization().x, p.getLocalization().y));
			}
			return retorno;
		}
		return null;
	}

	private void transformToShapesDiagram() {
		if(simulacao!=null)
		{
			this.diagram.setNome(simulacao.getNome());
			this.diagram.setPerfilProfundidade(simulacao.getMostrarPerfil());
			this.diagram.setIsShowDlgDetalhes(simulacao.getMostrarDetalhes());
			this.diagram.setIntervalo(simulacao.getIntervalo());

			for(Street2D obj : simulacao.getStreetsArray())
			{
				Street2DShape oStreet = (Street2DShape)this.transformStreet2DToStreet2DShape(obj);
				this.diagram.addChild(oStreet);
			}
			
			if(simulacao.getAgent()!=null)
			{
				Agent2DShape obs = (Agent2DShape)this.transformAgent2DToAgent2DShape((Agent2D)simulacao.getAgent());
				this.diagram.addChild(obs);
				for(Ponto2DShape p : obs.getPoints())
				{
					this.diagram.addChild(p);	
				}
			}
			
			for(IEntidadeMovel obj : simulacao.getObjetos())
			{
				if (obj instanceof Vehicle2D)
				{
					Vehicle2DShape o = (Vehicle2DShape)this.transformVehicle2DToVehicle2DShape((Vehicle2D)obj);
					this.diagram.addChild(o);
					
					for(Ponto2DShape p : o.getPoints())
					{
						this.diagram.addChild(p);	
					}
				}
				else if (obj instanceof Pedestrian2D)
				{
					Pedestrian2DShape o = (Pedestrian2DShape)this.transformPedestrian2DToPedestrian2DShape((Pedestrian2D)obj);
					this.diagram.addChild(o);
					
					for(Ponto2DShape p : o.getPoints())
					{
						this.diagram.addChild(p);	
					}
					
				}
			}
		}
	}

	private Shape transformPedestrian2DToPedestrian2DShape(Pedestrian2D obj) {
		if(obj!=null)
		{
			Pedestrian2DShape retorno = new Pedestrian2DShape();
			retorno.setParent(this.getDiagram());
			retorno.setNome(obj.getNome());
			retorno.setLocalization(new Point(obj.getX(), obj.getY()));
			retorno.setSize(new Dimension(obj.getWidth()*2, obj.getHeight()*2));
			retorno.setVelocity(obj.getVelocidade());
				
			for(java.awt.Point pAux : obj.getPontos())
			{
				Ponto2DShape pt = new Ponto2DShape();
				pt.setLocalization(new Point(pAux.x, pAux.y));
				pt.setParent(retorno);
				pt.setNomePai(retorno.getName());
				retorno.getPoints().add(pt);
			}
			return retorno;
		}
		return null;
	}
	
	private Shape transformStreet2DToStreet2DShape(Street2D oStreet) {
		if(oStreet!=null)
		{
			Street2DShape retorno = new Street2DShape();
			retorno.setParent(this.getDiagram());
			retorno.setNome(oStreet.getNome());
			retorno.setLocalization(new Point(oStreet.getX(), oStreet.getY()));
			retorno.setSize(new Dimension(oStreet.getWidth(), oStreet.getHeight()));
			return retorno;
		}
		return null;
	}
	
	// TODO Preparar para que seja possivel tranformar o vehicle em uma entidade
	// para o xml... por enquanto a entidade nao existe!
	private Shape transformVehicle2DToVehicle2DShape(Vehicle2D obj) {
		if(obj!=null)
		{
			Vehicle2DShape retorno = new Vehicle2DShape();
			retorno.setParent(this.getDiagram());
			retorno.setNome(obj.getNome());
			retorno.setLocalization(new Point(obj.getX(), obj.getY()));
			retorno.setSize(new Dimension(obj.getWidth(), obj.getHeight()));
			retorno.setSentido(obj.getSentido());
				
			for(java.awt.Point pAux : obj.getPontos())
			{
				Ponto2DShape pt = new Ponto2DShape();
				pt.setLocalization(new Point(pAux.x, pAux.y));
				pt.setParent(retorno);
				pt.setNomePai(retorno.getName());
				retorno.getPoints().add(pt);
			}
			return retorno;
		}
		return null;
	}

	private Shape transformAgent2DToAgent2DShape(Agent2D obs) {
		if(obs!=null)
		{
			Agent2DShape retorno = new Agent2DShape();
			retorno.setParent(this.getDiagram());
			retorno.setAngulo(obs.getAngulo());
			retorno.setLocalization(new Point(obs.getX(), obs.getY()));
			retorno.setNome(obs.getNome());
			retorno.setProfundidade(obs.getProfundidade());
			retorno.setSize(new Dimension(obs.getWidth(), obs.getHeight()));
			retorno.setVelocity(obs.getVelocidade());
				
			for(java.awt.Point pAux : obs.getPontos())
			{
				Ponto2DShape pt = new Ponto2DShape();
				pt.setLocalization(new Point(pAux.x, pAux.y));
				pt.setParent(retorno);
				pt.setNomePai(retorno.getName());
				retorno.getPoints().add(pt);
			}
			return retorno;
		}
		return null;
	}

	public ShapesDiagram getDiagram() {
		return diagram;
	}

	public Simulacao getSimulacao() {
		return simulacao;
	}
}