package br.edu.fei.dcc.raciocinioespacial.editor.shapes;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.actions.ActionFactory;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.actions.CreateNewPointAction;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.actions.RotateShapeAction;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;

/**
 * Prov as aes do menu de contexto do editor de formas.
 */
class ShapesEditorContextMenuProvider extends ContextMenuProvider {

	/** Registro de aes do editor. */
	private ActionRegistry actionRegistry;

	/**
	 * Instancia um novo provedor de menu de contexto para a EditPartViewer especfica e o ActionRegistry.
	 * @param viewer	A viso do editor grfico.
	 * @param registry	O ActionRegistry do editor.
	 * @throws IllegalArgumentException se o registro for <tt>nulo</tt>. 
	 */
	public ShapesEditorContextMenuProvider(EditPartViewer viewer, ActionRegistry registry, ShapesDiagram diagram) {
		super(viewer);
		if (registry == null) {
			throw new IllegalArgumentException();
		}
		actionRegistry = registry;
		this.actionRegistry.registerAction(new CreateNewPointAction(diagram, viewer));
		//this.actionRegistry.registerAction(new RotateShapeAction(diagram, viewer));
	}

	/**
	 * Chamado quando o menu de contexto  mostrado. Aes, 
	 * na qual o estado  habilitado, aparecero no menu de contexto.
	 * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	public void buildContextMenu(IMenuManager menu) {
		// Adicionam os grupos de ao padro ao menu.
		GEFActionConstants.addStandardActionGroups(menu);

		// Adiciona aes para o menu.
		menu.appendToGroup(
				GEFActionConstants.GROUP_UNDO, 			// Caminho do identificador do grupo
				getAction(ActionFactory.UNDO.getId())); // Aes a serem adicionadas.
		menu.appendToGroup(
				GEFActionConstants.GROUP_UNDO, 
				getAction(ActionFactory.REDO.getId()));
		menu.appendToGroup(
				GEFActionConstants.GROUP_EDIT,
				getAction(ActionFactory.DELETE.getId()));
		getAction(CreateNewPointAction.ID).setEnabled(getAction(ActionFactory.DELETE.getId()).isEnabled());
		menu.appendToGroup(
				GEFActionConstants.GROUP_EDIT,
				getAction(CreateNewPointAction.ID));
		/*getAction(RotateShapeAction.ID).setEnabled(getAction(ActionFactory.DELETE.getId()).isEnabled());
		menu.appendToGroup(
				GEFActionConstants.GROUP_EDIT,
				getAction(RotateShapeAction.ID));*/

	}

	private IAction getAction(String actionId) {
		return actionRegistry.getAction(actionId);
	}
}
