package br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements;
import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericProperty;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.properties.NumericPropertyTypeEnum;


/**
 * The shape for any mobile entity.
 */
public abstract class Mobile2DShape extends Shape{

	private static final String Proprerty_Velocity = "shapes.velocity";
	private static final long serialVersionUID = 1;
	private float velocity;
	private List<Ponto2DShape> points;
	
	@Override
	/**
	 * Creates all editable properties for design time
	 */
	protected void criaPropriedades() {
		this.getDescriptors().add(new NumericProperty(Proprerty_Velocity, "Velocity", NumericPropertyTypeEnum.FloatVelociry));
		super.criaPropriedades();
	}
	
	/** Returns the image icon used to represent the shape */
	public abstract Image getIcon();

	@Override
	public Object getPropertyValue(Object propertyId) {
		if (Proprerty_Velocity.equals(propertyId)) {
			return Float.toString(this.getVelocity());
		}
		return super.getPropertyValue(propertyId);
	}

	/** Returns the constant velocity defined by user for the shape */
	public float getVelocity() {
		return velocity;
	}
	
	/**
	 * We overload the method to define other size treatment.
	 */
	public void setSize(Dimension newSize) {
		if (newSize != null) {
			this.size.width = newSize.width;
			this.size.height = newSize.height;
			firePropertyChange(Proprerty_X, null, this.size);
		}
	}

	@Override
	/** Initiate the object with default configuration */
	protected void inicia() {
		super.inicia();
		this.points = new Vector<Ponto2DShape>();
		this.velocity = 0.5f;
		this.name = "Ped";
	}

	@Override
	/** Set property values */
	public void setPropertyValue(Object propertyId, Object value) {
		if (Proprerty_Velocity.equals(propertyId)) {
			float velocity = Float.parseFloat((String)value);
			this.setVelocity(velocity);
		}
		super.setPropertyValue(propertyId, value);
	}

	/** Set the constant velocity defined by user for the object */
	public void setVelocity(float velocity) {
		this.velocity = velocity;
	}

	/** Returns the name typed by user for the object */
	public String toString() {
		return this.getName();
	}

	/** Returns the route points defined by user for the shape */
	public List<Ponto2DShape> getPoints() {
		return points;
	}

	/** Set the route points defined by user for the shape */
	public void setPoints(List<Ponto2DShape> points) {
		this.points = points;
	}
}