package br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Agent2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Mobile2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Pedestrian2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Ponto2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Street2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Vehicle2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ModelElement;

/**
 * EditPart usada para instncias de Shape(mais especificadamente para EllipticalShape e
 * RectangularShape).
 * <p>Este edit part deve implementar a interface PropertyChangeListener, 
 * sendo assim, ser notificada quando houverem mudanas nas propriedades no model element correspondente.
 * </p>
 */
public class ShapeEditPart extends AbstractGraphicalEditPart implements PropertyChangeListener, NodeEditPart {

	private ConnectionAnchor anchor;

	/**
	 * Aps a ativao,  anexado no model element como um listener de modificaes das propriedades.
	 */
	public void activate() {
		if (!isActive()) {
			super.activate();
			((ModelElement) getModel()).addPropertyChangeListener(this);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// Permite a remoo de elementos do modelo associados.
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ShapeComponentEditPolicy());
	}

	/*(non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		IFigure f = createFigureForModel();
		if (getModel() instanceof Agent2DShape) {
			f.setOpaque(true); // non-transparent figure
			f.setBackgroundColor(ColorConstants.yellow);
		}
		else if (getModel() instanceof Street2DShape) {
			f.setOpaque(true); // non-transparent figure
			f.setBackgroundColor(ColorConstants.darkGray);
			f.setForegroundColor(ColorConstants.darkGray);
			f.setBounds(new Rectangle());
		}
		else if (getModel() instanceof Vehicle2DShape) {
			f.setOpaque(true); // non-transparent figure
			f.setBackgroundColor(ColorConstants.white);
		}
		else if (getModel() instanceof Pedestrian2DShape) {
			f.setOpaque(true); // non-transparent figure
			f.setBackgroundColor(ColorConstants.white);
		}
		else if(getModel() instanceof Ponto2DShape)
		{
			f.setBackgroundColor(ColorConstants.cyan);
			f.setForegroundColor(ColorConstants.cyan);
		}
		
		return f;
	}

	/**
	 * Retorna uma IFigure dependendo da instncia do elemento do modelo corrente.
	 * Isto permite que a EditPart possa ser usada nas subclasses de shape. 
	 */
	private IFigure createFigureForModel() {
		if (getModel() instanceof Vehicle2DShape) {
			return new RectangleFigure();
		}else if (getModel() instanceof Street2DShape) {
			return new RectangleFigure();
		}else if (getModel() instanceof Shape) {
			return new Ellipse();
		}
		else {
			throw new IllegalArgumentException();
		}
	}

	/**
	 * Aps a desativao,  desanexado do model element como um listener de modificaes das propriedades.
	 */
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((ModelElement) getModel()).removePropertyChangeListener(this);
		}
	}

	private Shape getCastedModel() {
		return (Shape) getModel();
	}

	protected ConnectionAnchor getConnectionAnchor() {
		if (anchor == null) {
			if (getModel() instanceof Mobile2DShape)
				anchor = new EllipseAnchor(getFigure());
			else if (getModel() instanceof Agent2DShape)
				anchor = new ChopboxAnchor(getFigure());
			else
				throw new IllegalArgumentException("Modelo invlido");
		}
		return anchor;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return getConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return getConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return getConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return getConnectionAnchor();
	}

	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		
		if(Shape.Proprerty_Name.equals(prop))
		{
			((DiagramEditPart) getParent()).renameChild((Shape)evt.getSource());
		}
		else
		{
			refreshVisuals();
		}
	}

	protected void refreshVisuals() {
		// Notifica ao container as alteraes de posio e localizao.
		// Necessrio para o container saber exatamente como renderizar os objetos.
		Rectangle bounds = new Rectangle(getCastedModel().getLocalization(),
				getCastedModel().getSize());
		((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), bounds);
	}
}