package br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractTreeEditPart;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ModelElement;

/**
 * TreeEditPart usado para instncias de forma/Shape (mais especificadamente para EllipticalShape e
 * RectangularShape). Isto  usado na viso 'Outline' do ShapesEditor.
 * <p>Este edit part deve implementar a interface PropertyChangeListener, 
 * sendo assim, ser notificada quando houverem mudanas nas propriedades no model element correspondente.
 * </p>
 */
public class ShapeTreeEditPart extends AbstractTreeEditPart implements PropertyChangeListener {

	/**
	 * Cria uma nova instncia para essa edit part usando os elementos do modelo.
	 * @param model uma instncia no-nula de Shape.
	 */
	ShapeTreeEditPart(Shape model) {
		super(model);
	}

	/**
	 * Aps a ativao,  anexado no model element como um listener de modificaes das propriedades.
	 */
	public void activate() {
		if (!isActive()) {
			super.activate();
			((ModelElement) getModel()).addPropertyChangeListener(this);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// Permite a remoo de elementos do modelo associados.
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ShapeComponentEditPolicy());
	}

	/**
	 * Aps a desativao,  desanexado do model element como um listener de modificaes das propriedades.
	 */
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((ModelElement) getModel()).removePropertyChangeListener(this);
		}
	}

	private Shape getCastedModel() {
		return (Shape) getModel();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	protected Image getImage() {
		return getCastedModel().getIcon();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getText()
	 */
	protected String getText() {
		return getCastedModel().toString();
	}

	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		refreshVisuals();
	}
}
