package br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;

/**
 * Fbrica que mapeia model elements para edit parts.
 */
public class ShapesEditPartFactory implements EditPartFactory {

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	public EditPart createEditPart(EditPart context, Object modelElement) {
		// obtm a EditPart do model element
		EditPart part = getPartForElement(modelElement);
		// armazena o model element no EditPart
		part.setModel(modelElement);
		return part;
	}

	/**
	 * Mapeia um objeto para um EditPart. 
	 * @throws RuntimeException se no forem encontrados resultados (erro de programao)
	 */
	private EditPart getPartForElement(Object modelElement) {
		if (modelElement instanceof ShapesDiagram) {
			return new DiagramEditPart();
		}
		if (modelElement instanceof Shape) {
			return new ShapeEditPart();
		}
		throw new RuntimeException(
				"No foi possvel criar um part para o model element: "
				+ ((modelElement != null) ? modelElement.getClass().getName() : "null"));
	}
}