%Number of frames that will be used in forecast, that means, if forecastFrames(10), new peaks will be asserted in 
%the following 5 time units. 
forecastFrames(30).
%Number of previous frames that will be used in forecast, that means, if forecastFrames(3), NumberOfFrames is 3...
%if forecastFrames(100), NumberOfFrames is 5
previousFrames(CurrentTime, NumberOfFrames):-
	CurrentTime >= 5,
	NumberOfFrames is 5 .
	
previousFrames(CurrentTime, NumberOfFrames):-
	CurrentTime =< 5,
	NumberOfFrames is CurrentTime.

%number of frames (time) in the future that will be analysed for collisions
collisionFrames(0, 15).	

%number of frames (time) in the future that will be analysed for overtaking
overtakingFrames(0, 15).

%number of frames (time) in the future that will be analysed for velocity reduction
breakingFrames(15, 30).
	
%Definition of peak
:- dynamic peak/5.

%Definition of situation
:- dynamic situation/3.

%Example of if-then-else...
ifthenelse((peak(X,_,_,_,Time)),Peak) :-
    TimeAnt is Time-1,
	(
	peak(X,_,_,_,TimeAnt)
    ->
	peak(X,Y,Z,W,TimeAnt),
	Peak = peak(X,Y,Z,W,TimeAnt)
    ;  
	peak(X,Y,Z,W,Time),
	Peak = peak(X,Y,Z,W,Time)
	).	

	
%virtual lines defined to detect if a peak is in colision area, overtaking area or safe area... 
%their parameters are A, B and C from straight line equation

%HFL
%old horizFrontLine(-7.5, -12.9903810567666, 0).
horizFrontLine(3, 5.19615242270663, 0).

%HBL
%old horizBackLine(-7.5, -12.9903810567666, -150).
horizBackLine(3, 5.19615242270663, 21.6).

%VOR
%old vertOuterRightLine(8.66025403784439, -5, -75).
vertOuterRightLine(3.11769145362398, -1.8, -10.8).

%VIR
%old vertInnerRightLine(8.66025403784439, -5, -25).
vertInnerRightLine(3.11769145362398, -1.8, -3.6).

%VIL
%old vertInnerLeftLine(8.66025403784439, -5, 25).
vertInnerLeftLine(3.11769145362398, -1.8, 3.6).

%VOL
%old vertOuterLeftLine(8.66025403784439, -5, 75).
vertOuterLeftLine(3.11769145362398, -1.8, 10.8).

%distance between lines (hardcoded to enhance performance)
horizDist(3.6).

vertOuterDist(2).

vertInnerDist(2).

%straight line equation (A, B, C), given coordinates from 2 points
straightLineEquation((X1, Y1), (X2,Y2), A, B, C):-
	A is Y1 - Y2,
	B is X2 - X1,
	C is X1 * Y2 - Y1 * X2.
	
%dintance from a point to a line
distancePointToLine((X,Y), A, B, C, Dist):-
	AuxAbs is A * X + B * Y + C,
	abs(AuxAbs, Abs),
	AuxSqr is A * A + B * B,
	sqrt(AuxSqr, Sqrt),
	Dist is Abs / Sqrt.
	
%Relative location of an object, in X and Y coordinates, assuming that the the viewpoint is at the origin (0,0)  
relativeLocation(Ang, Dist, (X, Y)):-
	RadAng is pi * Ang / 180,
	sin(RadAng, Sin),
	cos(RadAng, Cos),
	X is Dist * Cos,
	Y is Dist * Sin.

%defines the corresponding quarter where the point is located - 1 (+,+), 2(-,+), 3(-,-), 4(+,-)
quarter(X,Y, 1):-
	X >= 0 ,
	Y >= 0 .
	
quarter(X, Y, 2):-
	X < 0,
	Y >= 0 .
	
quarter(X, Y, 3):-
	X =< 0,
	Y < 0 .
	
quarter(X, Y, 4):-
	X > 0,
	Y < 0 .
	
	
%Angular location and distance of a object, given theis relative X and Y, assuming that the the viewpoint is at the origin (0,0)	
angularLocation((X, Y), Ang, Dist):-
	Square is X**2 + Y**2,
	sqrt(Square, Dist),
	AuxSin is Y / Dist,
	asin(AuxSin, RadAng),
	AuxAng is 180 * RadAng / pi,
	(
		quarter(X, Y, 1) ->
		Ang is AuxAng;
		(
			quarter(X, Y, 2) ->
			Ang is 180 - AuxAng;
				(
					quarter(X, Y, 3)->
					Ang is 180 - AuxAng;
					quarter(X, Y, 4),
					Ang is 360 + AuxAng
				)
		)
	).

%Obtain the Size, given an Angular Diameter (degrees) and the Depth (distance) 
physicalSize(AngDiam, Depth, Size):-
	Size is AngDiam * Depth * (2 * pi / 360).
	
%Obtain the Angular Diameter, given a Size and the Depth (distance)
angularDiameter(Size, Depth, AngDiam):-
	AngDiam is (Size / Depth) * (360 / (2 * pi)).
	
%Obtain the depth(distance) from the origin to the center of a body, given its depth to its closest point to 
%the origin (center depth - radius from the object) and angular diameter
getSizeAndCenterDepth(Depth, AngDiam, Size, CenterDepth):-
	SinAngDiamDiv2Rad = sin((AngDiam/2) * (pi/180)),
	Num is 2 * Depth * SinAngDiamDiv2Rad ,
	Denom is 1 - SinAngDiamDiv2Rad,
	Size is (Num / Denom),
	CenterDepth is Depth + (Size / 2).

%distance from a peak to every line
distancesPeakToDefinedLines(Peak, HFL, HBL, VOR, VIR, VIL, VOL):-
	peak(_, Depth, Size, Ang, _) = Peak,
	Radius is Size/ 2,
	relativeLocation(Ang, Depth, (X, Y)),
	horizFrontLine(A_HFL, B_HFL, C_HFL),
	distancePointToLine((X,Y), A_HFL, B_HFL, C_HFL, DistHFL),
	HFL is Radius-DistHFL,
	horizBackLine(A_HBL, B_HBL, C_HBL),
	distancePointToLine((X,Y), A_HBL, B_HBL, C_HBL, DistHBL),
	HBL is Radius-DistHBL,
	vertOuterRightLine(A_VOR, B_VOR, C_VOR),
	distancePointToLine((X,Y), A_VOR, B_VOR, C_VOR, DistVOR),
	VOR is Radius-DistVOR,
	vertInnerRightLine(A_VIR, B_VIR, C_VIR),
	distancePointToLine((X,Y), A_VIR, B_VIR, C_VIR, DistVIR),
	VIR is Radius-DistVIR,
	vertInnerLeftLine(A_VIL, B_VIL, C_VIL),
	distancePointToLine((X,Y), A_VIL, B_VIL, C_VIL, DistVIL),
	VIL is Radius-DistVIL,
	vertOuterLeftLine(A_VOL, B_VOL, C_VOL),
	distancePointToLine((X,Y), A_VOL, B_VOL, C_VOL, DistVOL),
	VOL is Radius-DistVOL.

oneGreaterThanZero(A, _):-
	A > 0 .
	
oneGreaterThanZero(_, B):-
	B > 0 .

%verify if there is a collision, given the distances from the object to the lines 
verifyCollision(HFL, HBL, VIR, VIL):-
	oneGreaterThanZero(HFL,HBL),
	oneGreaterThanZero(VIR,VIL).

verifyCollision(HFL, HBL, VIR, VIL):-
	horizDist(Dist),
	- Dist < HFL + HBL,
	oneGreaterThanZero(VIR, VIL).
	
verifyCollision(HFL, HBL, VIR, VIL):-
	vertInnerDist(Dist),
	- Dist < VIR + VIL, 
	oneGreaterThanZero(HFL, HBL).

verifyCollision(HFL, HBL, VIR, VIL):-
	horizDist(HDist),
	vertInnerDist(VDist),
	- HDist < HFL + HBL,
	- VDist < VIR + VIL.
	
	
%verify if an overtaking is unsafe, given the distances from the object to the lines
verifyUnsafeOvertaking(HFL, HBL, _, _, VOR, VOL):-
	oneGreaterThanZero(HFL,HBL),
	oneGreaterThanZero(VOR,VOL).

verifyUnsafeOvertaking(HFL, HBL, _, _, VOR, VOL):-	
	horizDist(Dist),
	- Dist < HFL + HBL,
	oneGreaterThanZero(VOR, VOL).
	
verifyUnsafeOvertaking(HFL, HBL, VIR, _, VOR, _):-	
	horizDist(Dist),
	- Dist < HFL + HBL,
	oneGreaterThanZero(VOR, VIR).
	
verifyUnsafeOvertaking(HFL, HBL, _, VIL, _, VOL):-	
	horizDist(Dist),
	- Dist < HFL + HBL,
	oneGreaterThanZero(VOL, VIL).

verifyUnsafeOvertaking(HFL, HBL, _, _, VOR, VOL):-
	vertInnerDist(Dist),
	- Dist < VOR + VOL, 
	oneGreaterThanZero(HFL, HBL).
	
verifyUnsafeOvertaking(HFL, HBL, _, VIL, _, VOL):-
	horizDist(HDist),
	vertInnerDist(VDist),
	- HDist < HFL + HBL,
	- VDist < VOL + VIL.
	
verifyUnsafeOvertaking(HFL, HBL, VIR, _, VOR, _):-
	horizDist(HDist),
	vertInnerDist(VDist),
	- HDist < HFL + HBL,
	- VDist < VOR + VIR.	

%True if a body is extending
extending(Body, CurrentTime):-
	peak(Body, Depth1, _, _, CurrentTime),
	PreviousTime is CurrentTime-1,
	peak(Body, Depth2, _, _, PreviousTime),
	Depth1 < Depth2,
	not(situation(Body, PreviousTime, vanishing)),
	retractall(situation(Body, CurrentTime, _)),
	assert(situation(Body, CurrentTime, extending)).
	
%True if a body is shrinking
shrinking(Body, CurrentTime):-
	peak(Body, Depth1, _, _, CurrentTime),
	PreviousTime is CurrentTime-1,
	peak(Body,Depth2, _, _, PreviousTime),
	Depth1 > Depth2,
	not(situation(Body, PreviousTime, vanishing)),
	retractall(situation(Body, CurrentTime, _)),
	assert(situation(Body, CurrentTime, shrinking)).

%True if a body is appearing	
appearing(Body, CurrentTime):-
	peak(Body, _, _, _, CurrentTime),
	PreviousTime is CurrentTime-1,
	not(peak(Body, _, _, _, PreviousTime)),
	not(situation(Body, PreviousTime, extending)),
	not(situation(Body, PreviousTime, shrinking)),
	not(situation(Body, PreviousTime, appearing)),
	retractall(situation(Body, CurrentTime, _)),
	assert(situation(Body, CurrentTime, appearing)).
	
%True if a body is vanishing
vanishing(Body, CurrentTime):-
	not(peak(Body, _, _, _, CurrentTime)),
	PreviousTime is CurrentTime-1,
	peak(Body, _, _, _, PreviousTime),
	not(situation(Body, PreviousTime, vanishing)),
	retractall(situation(Body, CurrentTime, _)),
	assert(situation(Body, CurrentTime, vanishing)).

%True if a body is stationary(keeping the same distance... does not check about its angle, only its distance)
stationary(Body, CurrentTime):-
	peak(Body, Depth1, _, _, CurrentTime),
	PreviousTime is CurrentTime-1,
	peak(Body, Depth2, _, _, PreviousTime),
	Depth1 = Depth2,
	not(situation(Body, PreviousTime, vanishing)),
	retractall(situation(Body, CurrentTime, _)),
	assert(situation(Body, CurrentTime, stationary)).	
	
%Returns the current situation of a body (that can be extending, shrinking, appearing, vanishing and stationary)
holdsAt(Body, CurrentTime, extending):-
	situation(Body, CurrentTime, extending).

holdsAt(Body, CurrentTime, shrinking):-
	situation(Body, CurrentTime, shrinking).
	
holdsAt(Body, CurrentTime, appearing):-
	situation(Body, CurrentTime, appearing).
	
holdsAt(Body, CurrentTime, vanishing):-
	situation(Body, CurrentTime, vanishing).

holdsAt(Body, CurrentTime, stationary):-
	situation(Body, CurrentTime, stationary).
	
holdsAt(Body, CurrentTime, extending):-
	extending(Body, CurrentTime).
	
holdsAt(Body, CurrentTime, shrinking):-
	shrinking(Body, CurrentTime).
	
holdsAt(Body, CurrentTime, appearing):-
	appearing(Body, CurrentTime).
	
holdsAt(Body, CurrentTime, vanishing):-
	vanishing(Body, CurrentTime).
	
holdsAt(Body, CurrentTime, stationary):-	
	stationary(Body, CurrentTime).
	
%We are not using this... but can be useful	
retriveHead([Head|[]], Head).
	
retriveHead([Head|_], Head).

%Given the Delta of X, Delta of Y and Delta of Size previous calculated, assert peaks of a body at their probable
%location in future instants, assuming that the body will keep it current movement
assertForecast(peak(_, _, _, _, _), _, _, _, 0 ).

assertForecast(peak(Body, Depth, Size, AngDist, CurrentTime), DeltaX, DeltaY, DeltaSize, Frames):-
	forecastFrames(AuxTime),
	FrameTime is AuxTime - Frames + 1,
	relativeLocation(AngDist, Depth, (X, Y)),
	FrameX is DeltaX * FrameTime + X,
	FrameY is DeltaY * FrameTime + Y,
	%old FrameSize is DeltaSize * FrameTime + Size,
	FrameSize is Size,
	angularLocation((FrameX, FrameY), FrameAng, FrameDist),
	ForecastTime is FrameTime + CurrentTime,
	retractall(peak(Body, _, _, _, ForecastTime)),
	assert(peak(Body, FrameDist, FrameSize, FrameAng, ForecastTime)),
	FollowingFrames is Frames-1,
	assertForecast(peak(Body, Depth, Size, AngDist, CurrentTime), DeltaX, DeltaY, DeltaSize, FollowingFrames).

%Retrive Delta of X, Delta of Y and Delta of Size from a object at the given instants
retriveDeltas(_, 0, DeltaX, DeltaY, DeltaSize, TotalX, TotalY, TotalSize,TotalFrames):-
	%peak(Body, Depth, Size, AngDist, CurrentTime) = Peak,
	DeltaX is 0,
	DeltaY is 0,
	DeltaSize is 0,
	TotalX is 0,
	TotalY is 0,
	TotalSize is 0,
	TotalFrames is 0 .
	
retriveDeltas(Peak, PreviousFrames, DeltaX, DeltaY, DeltaSize, TotalX, TotalY, TotalSize, TotalFrames):-
	peak(Body, Depth, Size, AngDist, CurrentTime) = Peak,
	PreviousTime is CurrentTime-1,
	(
		peak(Body, PrevDepth, PrevSize, PrevAngDist, PreviousTime)
		->
		PrevPeak = peak(Body, PrevDepth, PrevSize, PrevAngDist, PreviousTime),
		AuxPreviousFrames is PreviousFrames - 1 ,
		retriveDeltas(PrevPeak, AuxPreviousFrames, PrevDeltaX, PrevDeltaY, PrevDeltaSize, PrevTotalX, PrevTotalY, PrevTotalSize, PrevTotalFrames)
		;
		peak(Body, PrevDepth, PrevSize, PrevAngDist, CurrentTime),
		PrevPeak = peak(Body, PrevDepth, PrevSize, PrevAngDist, CurrentTime),
		retriveDeltas(PrevPeak, 0, PrevDeltaX, PrevDeltaY, PrevDeltaSize, PrevTotalX, PrevTotalY, PrevTotalSize, PrevTotalFramesAux),
		PrevTotalFrames is PrevTotalFramesAux - 1
	),
	relativeLocation(AngDist, Depth, (X,Y)),
	relativeLocation(PrevAngDist, PrevDepth, (PrevX,PrevY)),
	TotalX is X-PrevX+PrevTotalX,
	TotalY is Y-PrevY+PrevTotalY,
	TotalSize is Size-PrevSize+PrevTotalSize,
	TotalFrames is PrevTotalFrames + 1,
	(
	TotalFrames is 0
	->
	DeltaX is TotalX,
	DeltaY is TotalY,
	DeltaSize is TotalSize
	;
	DeltaX is TotalX / TotalFrames,
	DeltaY is TotalY / TotalFrames,
	DeltaSize is TotalSize / TotalFrames
	).

	
	
%To all objects, retrive their deltas and assert their position in future instants
prepareAssertForecast([]).
	
prepareAssertForecast([Peak|PeakList]):-
	prepareAssertForecast(PeakList),
	peak(Body, _, _, _, CurrentTime) = Peak,
	previousFrames(CurrentTime, PreviousFrames),
	retractall(situation(Body, CurrentTime, _)),
	(
		holdsAt(Body, CurrentTime, extending) ->
		retriveDeltas(Peak, PreviousFrames, DeltaX, DeltaY, DeltaSize, _, _, _, _),
		forecastFrames(Frames),
		assertForecast(Peak, DeltaX, DeltaY, DeltaSize, Frames);
		true
	).
	
%Find all objects of the current perception and use the prepareAssertForecast axiom...
forecast(CurrentTime):-
	findall((peak(Body,Depth,Size,AngDist,CurrentTime)), peak(Body,Depth,Size,AngDist,CurrentTime), PeakList),
	prepareAssertForecast(PeakList).
	
%This axiom removes the 't' char from the beginning of the atom, returning its number
getTime(Time, TimeNumber):-
	atom_chars(Time,[_|Tail]),
	number_chars(TimeNumber,Tail).
	
%This function return the angular distance adding every description size of the list tail before the head 
%description, assuming that the viewpoint right border angle is 0 degree... 
getAngDist([], 0).
	
getAngDist([background( _, _, Size)|Tail], TotalSize):-
	getAngDist(Tail, TotalSize1),
	TotalSize is TotalSize1 + Size.
	
getAngDist([foreground( _, _, Size)|Tail], TotalSize):-
	getAngDist(Tail, TotalSize1),
	TotalSize is TotalSize1 + Size.

%The function description assert every peak(Body,Depth,Size,AngDist,Time) found inside the given list.
description(_, []).
	
description(Time, [foreground(Body,Depth,AngDiam)|Tail]):-
	description(Time, Tail),
	getAngDist([foreground(Body,Depth,AngDiam)|Tail], AngDist1),
	AngDist is AngDist1 - AngDiam/2,
	getTime(Time, NumberTime), % remove the 't' from the beginning of the time
	getSizeAndCenterDepth(Depth, AngDiam, Size, CenterDepth),
	assert(peak(Body,CenterDepth,Size,AngDist, NumberTime)),
	retractall(situation(Body, NumberTime, _)),
	holdsAt(Body, NumberTime, _).
	
description(Time, [background(_,_,_)|Tail]):-
	description(Time, Tail). % background doesn't have much information...

%iterates through every peak from a given time and checks if there is collision
queryObjectInColisionRouteLoopPeak([], _).

queryObjectInColisionRouteLoopPeak([situation(Body, _, _)|Tail], Time):-
	peak(Body, Depth, Size, AngDist, Time),
	Peak = peak(Body, Depth, Size, AngDist, Time),
	distancesPeakToDefinedLines(Peak, HFL, HBL, _, VIR, VIL, _),
	not(verifyCollision(HFL, HBL, VIR, VIL)),
	queryObjectInColisionRouteLoopPeak(Tail, Time).

%iterates through every time in the future from the current times, checking for collision	
queryObjectInColisionRouteLoopTime(_, InitTime, EndTime):-
	InitTime > EndTime.
	
queryObjectInColisionRouteLoopTime(SituationList, InitTime, EndTime):-
	EndTime >= InitTime,
	(
		queryObjectInColisionRouteLoopPeak(SituationList, InitTime)->
		InitTime1 is InitTime + 1,
		queryObjectInColisionRouteLoopTime(SituationList, InitTime1, EndTime);
		fail
	).

%checks in the defined future time and every peak, if there is a collision
queryObjectInColisionRoute(Time) :- 
	findall((situation(Body, Time, extending)), situation(Body, Time, extending), SituationList),
	collisionFrames(InitTime1, EndTime1),
	InitTime is Time + InitTime1,
	EndTime is Time + EndTime1,
	not(queryObjectInColisionRouteLoopTime(SituationList, InitTime, EndTime)).
	
%iterates through every peak from a given time and checks if it is a unsafe overtaking situation
queryUnsafeOvertakingLoopPeak([], _).

queryUnsafeOvertakingLoopPeak([situation(Body, _, _)|Tail], Time):-
	peak(Body, Depth, Size, AngDist, Time),
	Peak = peak(Body, Depth, Size, AngDist, Time),
	distancesPeakToDefinedLines(Peak, HFL, HBL, VOR, VIR, VIL, VOL),
	not(verifyUnsafeOvertaking(HFL, HBL, VIR, VIL, VOR, VOL)),
	queryUnsafeOvertakingLoopPeak(Tail, Time).

%iterates through every time in the future from the current times, checking if it is a unsafe overtaking situation	
queryUnsafeOvertakingLoopTime(_, InitTime, EndTime):-
	InitTime > EndTime.
	
queryUnsafeOvertakingLoopTime(SituationList, InitTime, EndTime):-
	EndTime >= InitTime,
	(
		queryUnsafeOvertakingLoopPeak(SituationList, InitTime)->
		InitTime1 is InitTime + 1,
		queryUnsafeOvertakingLoopTime(SituationList, InitTime1, EndTime);
		fail
	).

%checks in the defined future time and every peak, if it is a unsafe overtaking situation
queryUnsafeOvertaking(Time) :- 
	findall((situation(Body, Time, extending)), situation(Body, Time, extending), SituationList),
	overtakingFrames(InitTime1, EndTime1),
	InitTime is Time + InitTime1,
	EndTime is Time + EndTime1,
	not(queryUnsafeOvertakingLoopTime(SituationList, InitTime, EndTime)).

%checks in the defined future time and every peak, if it is a velocity reduction needed situation
queryVelocityReduceNeeded(Time) :- 
	findall((situation(Body, Time, extending)), situation(Body, Time, extending), SituationList),
	breakingFrames(InitTime1, EndTime1),
	InitTime is Time + InitTime1,
	EndTime is Time + EndTime1,
	not(queryObjectInColisionRouteLoopTime(SituationList, InitTime, EndTime)).

%performs the assert of a peak reiceived from java
performFunction(0, FunctionArguments, Response) :- 
    description(Time, _) = FunctionArguments,
	getTime(Time, TimeNumber),
	retractall(peak(_, _, _, _, TimeNumber)),
	(
		FunctionArguments->
		forecast(TimeNumber),
		Response = 'true';
		Response = 'false'
	),
	%clean up... erases all peaks from the past that will be not "used"... improves performance
	previousFrames(TimeNumber, NumberOfFrames),
	CleanUpTime is TimeNumber - NumberOfFrames - 1,
	retractall(situation(_, CleanUpTime, _)),
	retractall(peak(_, _, _, _, CleanUpTime)).

%checks from collision in the given time, reiceived from java
performFunction(1, FunctionArguments, Response) :-
    (
	queryObjectInColisionRoute(FunctionArguments)->
	Response = 'true';
	Response = 'false'
	).

%checks from unsafe overtaking situation in the given time, reiceived from java	
performFunction(2, FunctionArguments, Response) :-
    (
	queryUnsafeOvertaking(FunctionArguments)->
	Response = 'true';
	Response = 'false'
	).
	
%checks from velocity reduction needed situation in the given time, reiceived from java
performFunction(3, FunctionArguments, Response) :-
    (
	queryVelocityReduceNeeded(FunctionArguments)->
	Response = 'true';
	Response = 'false'
	).
	
%end the connection with java
performFunction(4, _, Response) :-
    Response = end_prolog.

%establishes a connection with the java
connect(Port) :- 
   tcp_socket(Socket), 
   gethostname(Host),  % local host
   tcp_connect(Socket,Host:Port),
   tcp_open_socket(Socket,INs,OUTs), 
   assert(connectedReadStream(INs)), 
   assert(connectedWriteStream(OUTs)).

:- connect(22105).

%receive and perform what is asked by java, and returns the results
spatialReasoning :- 
   connectedReadStream(IStream), 
   read(IStream,(Function, RequestNumber, FunctionArguments)),
   performFunction(Function, FunctionArguments, Response), 
   connectedWriteStream(OStream), 
   write(OStream, (RequestNumber, Response)), 
   nl(OStream), flush_output(OStream),
   spatialReasoning.

:- spatialReasoning.