/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.movimentacao;

/**
 * @author au.edu.unsw.cse
 *
 */

/**Classe que define uma curva formada por uma seqncia de pontos de controle */
import java.awt.Polygon;

public class ControlCurve {

	private static final int EPSILON = 36;  /* square of distance for picking */
	protected Polygon pts;
	protected int selection = -1;
	protected int steps = 180;

	public ControlCurve() {
		pts = new Polygon();
	}

	/** add a control point, return index of new control point */
	public int addPoint(int x, int y) {
		pts.addPoint(x,y);
		return selection = pts.npoints - 1;
	}

	/** remove selected control point */
	public void removePoint() {
		if (selection >= 0) {
			pts.npoints--;
			for (int i = selection; i < pts.npoints; i++) {
				pts.xpoints[i] = pts.xpoints[i+1];
				pts.ypoints[i] = pts.ypoints[i+1];
			}
		}
	}

	/** return index of control point near to (x,y) or -1 if nothing near */
	public int selectPoint(int x, int y) {
		int mind = Integer.MAX_VALUE;
		selection = -1;
		for (int i = 0; i < pts.npoints; i++) {
			int d = ControlCurve.sqr(pts.xpoints[i]-x) + 
			ControlCurve.sqr(pts.ypoints[i]-y);
			if (d < mind && d < EPSILON) {
				mind = d;
				selection = i;
			}
		}
		return selection;
	}

	/** set selected control point */
	public void setPoint(int x, int y) {
		if (selection >= 0) {
			pts.xpoints[selection] = x;
			pts.ypoints[selection] = y;
		}
	}

	private static int sqr(int x)
	{
		return (int)ControlCurve.sqr((double)x);
	}
	
	private static double sqr(double x)
	{
		return Math.pow(x, x);
	}
	
}

