package br.edu.fei.dcc.raciocinioespacial.simulador.movimentacao.colisao;

import java.util.ArrayList;
import java.util.Arrays;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.IEntidade2D;

public class ColisaoPotencial {
	private ArrayList<IEntidade2D> candidatas;
	private IEntidade2D entidade;
	public ColisaoPotencial(IEntidade2D entidade) {
		this(entidade, new ArrayList<IEntidade2D>());
	}
	
	public ColisaoPotencial(IEntidade2D entidade, IEntidade2D[] candidatas) {
		this(entidade, new ArrayList<IEntidade2D>(Arrays.asList(candidatas)));
	}
	
	public ColisaoPotencial(IEntidade2D entidade, IEntidade2D candidata) {
		super();
		this.entidade = entidade;
		this.candidatas = new ArrayList<IEntidade2D>();
		this.getCandidatas().add(candidata);
	}
	
	public ColisaoPotencial(IEntidade2D entidade, ArrayList<IEntidade2D> candidatas) {
		super();
		this.entidade = entidade;
		this.setCandidatas(candidatas);
	}
	
	public ArrayList<IEntidade2D> getCandidatas() {
		return candidatas;
	}
	public IEntidade2D getEntidade() {
		return entidade;
	}
	public void setCandidatas(ArrayList<IEntidade2D> candidatas) {
		if(candidatas!=null)
		{
			this.candidatas = new ArrayList<IEntidade2D>();
			for(IEntidade2D e : candidatas)
			{
				if(e!=this.entidade) this.candidatas.add(e);
			}
		}
	}
	public void setEntidade(IEntidade2D entidade) {
		this.entidade = entidade;
	}
	
	public static IEntidade2D[][] listaColisaoPotencialToIEntidade2DArray(ArrayList<ColisaoPotencial> colisoes)
	{
		IEntidade2D[][] retorno = null;
		if(colisoes!=null)
		{
			retorno = new IEntidade2D[colisoes.size()][];
			for(int i=0; i<colisoes.size(); i++)
			{
				retorno[i] = new IEntidade2D[colisoes.get(i).getCandidatas().size()+1];
				colisoes.get(i).getCandidatas().toArray(retorno[i]);
				retorno[i][retorno[i].length-1] = colisoes.get(i).getEntidade();
			}
		}
		return retorno;
	}
	
}