package br.edu.fei.dcc.raciocinioespacial.simulador.persistencia;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import br.edu.fei.dcc.raciocinioespacial.simulador.simulacao.Simulacao;

import com.thoughtworks.xstream.XStream;

public class PersistenciaSimulacao implements IPersistencia{
	
	private String nomeArquivo;
	private Simulacao simulacao;
	
	public PersistenciaSimulacao(String nomeArquivo) {
		super();
		this.nomeArquivo = nomeArquivo;
	}

	public PersistenciaSimulacao(String nomeArquivo, Simulacao simulacao) {
		super();
		this.nomeArquivo = nomeArquivo;
		this.simulacao = simulacao;
	}

	public String getNomeArquivo() {
		return nomeArquivo;
	}

	public Simulacao getSimulacao() {
		return simulacao;
	}
	
	public void setNomeArquivo(String nomeArquivo) {
		this.nomeArquivo = nomeArquivo;
	}		
	
	public void setSimulacao(Simulacao simulacao) {
		this.simulacao = simulacao;
	}

	private void escreverArquivo(String texto){
		try  
		{ 
			BufferedWriter escreve = new BufferedWriter(new FileWriter(this.getNomeArquivo()));
			escreve.write(texto);
			escreve.flush ();
			escreve.close ();
		}
		catch(IOException e){}
	}
	
	private StringBuilder lerArquivo(){
		StringBuilder texto = new StringBuilder();
		
		try {
			BufferedReader in = new BufferedReader(new FileReader(this.getNomeArquivo()));
	        
			while (in.ready()) {
				texto.append(in.readLine());
			}
			in.close();
			return texto;
		}
		catch (IOException e) {
			return null;
		}
	}
	
	public Simulacao obter() throws Exception{
		if(this.getNomeArquivo()==null){
			throw new Exception("Para abrir um arquivo,  necessrio preencher o nome");
		}
		StringBuilder strb = new StringBuilder(); 
		
		strb = this.lerArquivo();
		
		XStream xstream = new XStream();
		xstream.alias("simulacao", Simulacao.class);
		Simulacao nova = (Simulacao)xstream.fromXML(strb.toString());
		return nova;			
	}
	
	public void salvar() throws Exception{
		if(this.getNomeArquivo()==null){
			throw new Exception("Para salvar um arquivo,  necessrio preencher o nome");
		}
		if(this.getSimulacao()==null){
			throw new Exception("Para salvar um arquivo,  necessria uma simulao");
		}
			
		XStream xstream = new XStream();
		xstream.alias("simulacao", Simulacao.class);
		
		String texto = xstream.toXML(simulacao);
		this.escreverArquivo(texto);
	} 
}