package br.edu.fei.dcc.raciocinioespacial.simulador.util;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import sun.swing.plaf.synth.Paint9Painter.PaintType;

public class Utils {
	
	private static final float CONST_SCALE = 0.2f;

	public static void aguardar(long milisegundos){
		try {
			Thread.sleep(milisegundos);
		} catch (InterruptedException e) {
			return;
		}
	}

	private static void configGraphics(Graphics g, Stroke s, Color c)
	{
		if(g==null) return;
		Graphics2D g2 = (Graphics2D)g;
		g2.setColor(c);
		g2.setStroke(s);
		
		RenderingHints rh = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		g2.setRenderingHints(rh);
	}
	
	public static void drawArc(Arc2D obj, Graphics g, Color c, Stroke s)
	{
		if(g==null)return;
		Utils.configGraphics(g, s, c);
		((Graphics2D)g).draw(obj);
	}
	
	public static void drawShape(RectangularShape obj,
			boolean isShowBorder, Graphics g, Color c, Stroke s)
	{
		if(g==null)return;
		
		if(isShowBorder)
		{
			Utils.configGraphics(g, s, Color.BLACK);
			((Graphics2D)g).draw(obj);
		}
		
		Utils.configGraphics(g, s, c);
		((Graphics2D)g).fill(obj);
	}
	
	/*public static void drawOval(int x, int y, int raio, boolean preenchido, Graphics g)
	{
		Utils.drawOval(new Ellipse2D.Double(x-raio,y-raio,raio*2,raio*2), preenchido, g);
	}*/

	public static float getAnguloInclinacaoReta(int x0, int y0, int x1, int y1)
	{
		if((x0-x1)!=0)
		{
			double m = (double)(y0-y1)/(x0-x1);
			return (float)Math.atan(m);
		}
		return 0.0f;
	}
	
	public static float getAnguloAgudoEntreRetas(int x0, int y0, int xInterseccao, int yInterseccao, int x1, int y1)
	{
		double m0 = (double)(y0-yInterseccao)/(x0-xInterseccao);
		double m1 = (double)(y1-yInterseccao)/(x1-xInterseccao);
		double tanAngulo = Math.abs((m0-m1)/(1+m0*m1));
		return (float)Math.atan(tanAngulo);
	}
	
	public static Quadrante getQuadranteRelativo(int xBase, int yBase, int x, int y) throws Exception
	{
		if((x>xBase)&&(y<yBase))
			return Quadrante.PRIMEIRO;
		else if((x<xBase)&&(y<yBase))
			return Quadrante.SEGUNDO;
		else if((x<xBase)&&(y>yBase))
			return Quadrante.TERCEIRO;
		else if((x>xBase)&&(y>yBase))
			return Quadrante.QUARTO;
		else
			throw new Exception(Messages.getString("Utils.Erro_Quadrante")); //$NON-NLS-1$
	}
	
	public static int quadrante2Int(Quadrante q)
	{
		if(q==Quadrante.PRIMEIRO)
			return 1;
		else if(q==Quadrante.SEGUNDO)
			return 2;
		else if(q==Quadrante.TERCEIRO)
			return 3;
		else if(q==Quadrante.QUARTO)
			return 4;
		else
			return -1;
	}
	
	public static String now() {
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat(Messages.getString("Utils.Formato_Completo_Hora")); //$NON-NLS-1$
		return sdf.format(cal.getTime());
	}
	
	public static Polygon removePolygonPointAt(Polygon p, int start, int end)
	{
		Polygon retorno = new Polygon();
		if(p!=null)
		{
			if((start<=p.npoints)&&(start>=0)&&(end<=p.npoints)&&(end>=start))
			{
				for(int i=0; i<p.npoints; i++)
				{
					if((i<start)||(i>end))
					{
						retorno.addPoint(p.xpoints[i], p.ypoints[i]);
					}
				}
			}
		}
		return retorno;
	}

	public static Polygon addPolygonPointAt(Polygon p, int index, int x, int y)
	{
		Polygon retorno = new Polygon();
		if(p!=null)
		{
			if((index>=0)&&(index<=(p.npoints+1)))
			{
				if(index==p.npoints)
				{
					p.addPoint(x, y);
					return p;
				}
				for(int i=0; i<p.npoints; i++)
				{
					if(i==index)
					{
						retorno.addPoint(x, y);
					}
					retorno.addPoint(p.xpoints[i], p.ypoints[i]);
				}
			}
		}
		return retorno;
	}
	
	public static List<Point> getPontosIntersecaoRetaCircunferencia(float coeficienteAngularReta, float coeficienteLinearReta, int XcCircunferencia, int YcCircunferencia, int raioCircunferencia)
	{
		List<Point> retorno = new ArrayList<Point>();
		if((coeficienteAngularReta==coeficienteLinearReta)&&(coeficienteAngularReta==0))
		{
			retorno.add(new Point(XcCircunferencia, XcCircunferencia+raioCircunferencia));
			retorno.add(new Point(XcCircunferencia, XcCircunferencia-raioCircunferencia));
		}else
		{
		
			double c1 = (1+Math.pow(coeficienteAngularReta, 2));
			double c2 = (2*(coeficienteAngularReta*coeficienteLinearReta))-(2*(XcCircunferencia))-(2*(YcCircunferencia*coeficienteAngularReta));
			double c3 = (coeficienteLinearReta*(coeficienteLinearReta-(2*YcCircunferencia)))+(Math.pow(XcCircunferencia, 2))+(Math.pow(YcCircunferencia, 2))-(Math.pow(raioCircunferencia, 2));
			
			double delta = (Math.pow(c2, 2))-4*c1*c3;
			
			if(delta<0)
			{
				return null;
			}else
			{
				double xPos = (((-1)*c2) + Math.sqrt(delta))/(2*c1);
				double yPos = coeficienteAngularReta*xPos+coeficienteLinearReta;
				double xNeg = (((-1)*c2) - Math.sqrt(delta))/(2*c1);
				double yNeg = coeficienteAngularReta*xNeg+coeficienteLinearReta;
				retorno.add(new Point((int)xPos, (int)yPos));
				retorno.add(new Point((int)xNeg, (int)yNeg));
			}
		}
		return retorno;
	}
	
	public static Point getPontoComMenorDistancia(int x0, int y0, List<Point> pontos)
	{
		if(pontos!=null)
		{
			Point retorno = null;
			for(Point p : pontos)
			{
				if(retorno==null)
					retorno = p;
				else
				{
					if((Point.distance(retorno.x, retorno.y, x0, y0))>(Point.distance(p.x, p.y, x0, y0)))
					{
						retorno = p;
					}
				}
			}
			return retorno;
		}
		return null;
	}
	
	public static void drawArrowLine(int x1, int y1, int x2, int y2, int w, int h, Graphics g)
	{
		g.drawLine(x1, y1, x2, y2);
		if(x1==x2)
		{
			g.drawLine(x2-w, y2-h, x2, y2);
			g.drawLine(x2+w, y2-h, x2, y2);
		}else if(y1==y2)
		{
			g.drawLine(x2-w, y2-h, x2, y2);
			g.drawLine(x2-w, y2+h, x2, y2);
		}
	}
	
	/** Converts a pixel measure in meter measure (constant scale for the sim 5px=1m)*/
	public static float convertPixelsToMeters(float fPixels)
	{
		return fPixels * CONST_SCALE;
	}
	
	public static float convertPixelsToMeters(double dPixels)
	{
		return convertPixelsToMeters((float)dPixels);
	}
	
	/** Converts a meter measure in pixel measure (constant scale for the sim 5px=1m)*/
	public static float convertMetersToPixels(float fMeters)
	{
		return fMeters / CONST_SCALE;
	}
}