/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.entidades;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Ellipse2D;
import java.awt.geom.RectangularShape;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;

import br.edu.fei.dcc.raciocinioespacial.simulador.util.SentidoVehicle;

/** 
 * ?
 */
public class Pedestrian2D extends Observable implements IEntidade2D {
	private static final long serialVersionUID = 1L;

	public static List<Pedestrian2D> transformEntidadeMovelToObjeto2D(List<IEntidadeMovel> in)
	{
		if(in!=null)
		{
			List<Pedestrian2D> retorno = new ArrayList<Pedestrian2D>();
			for(IEntidadeMovel x : in)
			{
				retorno.add((Pedestrian2D)x);
			}
			return retorno;
		}
		return null;
	}
	
	protected SentidoVehicle svSentido = SentidoVehicle.VERTICAL;

	private Color cor;
	
	private String nome;
	
	protected RectangularShape obj2DOld;

	//private Polygon percurso;

	private ArrayList<Point> pontos;
	
	protected boolean isMoving = false;
	protected boolean isPosIncreasing = false;

	private float velocidade;	

	private int x;
	private int y;
	
	protected float xCurrent = 0f;
	protected float yCurrent = 0f;
	protected int currentPoint = -1;
	
	protected int iHeight;
	protected int iWidth;
	
	public Pedestrian2D()
	{
		this(0,0,0,10,"obj");
	}
	
	public Pedestrian2D(int x, int y, int width, int height, String nome){
		this(x,y,width,height,nome, Color.ORANGE);
	}

	public Pedestrian2D(int x, int y, int width, int height, String nome, Color cor){
		this(x,y,width, height,nome,cor, new ArrayList<Point>());
	}

	public Pedestrian2D(int x, int y, int width, int height, String nome, Color cor, ArrayList<Point> pontos){
		this.setX(x);
		this.setY(y);
		this.setWidth(width);
		this.setHeight(height);
		this.setNome(nome);
		this.setCor(cor);
		this.setPontos(pontos);
	}

	/* (non-Javadoc)
	 * @see br.edu.fei.dcc.raciocionioespacial.simulador.entidades.IEntidade#criarEntidade()
	 */
	@Override
	public IEntidade criarEntidade() {
		return this;
	}

	/**
	 * Getter of the property <tt>cor</tt>
	 * @return  Returns the cor.
	 * @uml.property  name="cor"
	 */
	public Color getCor() {
		return cor;
	}

	/**
	 * Getter of the property <tt>nome</tt>
	 * @return  Returns the nome.
	 * @uml.property  name="nome"
	 */
	public String getNome() {
		return nome;
	}
	
	public RectangularShape getObjeto2D()
	{
		obj2DOld = new Ellipse2D.Float(xCurrent, yCurrent, this.getWidth(), this.getHeight());
		return obj2DOld;
	}

	/**
	 * Getter of the property <tt>percurso</tt>
	 * @return  Returns the percurso.
	 * @uml.property  name="percurso"
	 */
	/*public Polygon getPercurso() {
		return percurso;
	}*/

	/**
	 * Getter of the property <tt>pontos</tt>
	 * @return  Returns the pontos.
	 * @uml.property  name="pontos"
	 */
	public ArrayList<Point> getPontos() {
		return pontos;
	}

	//@Override
	/*public Point getPosicao(float t) {
		int passo = this.getPasso(t);
		return new Point(this.getPercurso().xpoints[passo], this.getPercurso().ypoints[passo]);
	}*/

	@Override
	public Rectangle getRect() {
		int dim = iWidth;
		return new Rectangle((int)xCurrent, (int)yCurrent, dim, dim);
	}
	
	/**
	 * Getter of the property <tt>velocidade</tt>
	 * @return  Returns the velocidade.
	 * @uml.property  name="velocidade"
	 */
	public float getVelocidade() {
		return velocidade;
	}
	
	/**
	 * Getter of the property <tt>x</tt>
	 * @return  Returns the x.
	 * @uml.property  name="x"
	 */
	public int getX() {
		return x;
	}

	/**
	 * Getter of the property <tt>y</tt>
	 * @return  Returns the y.
	 * @uml.property  name="y"
	 */
	public int getY() {
		return y;
	}

	public void notificarObservadores(int instanteAnterior, int instanteAtual, float instante)
	{
		this.notifyObservers(null);
	}
	
	@Override
	public void notifyObservers(Object arg) {
		this.setChanged();
		super.notifyObservers(arg);
	}

	/**
	 * Setter of the property <tt>cor</tt>
	 * @param cor  The cor to set.
	 * @uml.property  name="cor"
	 */
	public void setCor(Color cor) {
		this.cor = cor;
	}

	/**
	 * Setter of the property <tt>nome</tt>
	 * @param nome  The nome to set.
	 * @uml.property  name="nome"
	 */
	public void setNome(String nome) {
		this.nome = nome;
	}

	/**
	 * Setter of the property <tt>percurso</tt>
	 * @param percurso  The percurso to set.
	 * @uml.property  name="percurso"
	 */
	/*public void setPercurso(Polygon percurso) {
		this.percurso = percurso;
		this.atualizaVelocidadeInstantanea();
	}*/
	
	/*private void atualizaVelocidadeInstantanea() {
		this.velocidadeInstantanea = new float[this.percurso.npoints];
		for(int i=0; i<this.velocidadeInstantanea.length; i++)
		{
			this.velocidadeInstantanea[i] = this.velocidade;
		}
		if(this.percurso.npoints>0)
			this.velocidadeInstantanea[this.percurso.npoints-1] = 0;
	}*/
	
	/**
	 * Setter of the property <tt>pontos</tt>
	 * @param pontos  The pontos to set.
	 * @uml.property  name="pontos"
	 */
	public void setPontos(ArrayList<Point> pontos) {
		this.pontos = pontos;
	}

	/**
	 * Setter of the property <tt>velocidade</tt>
	 * @param velocidade  The velocidade to set.
	 * @uml.property  name="velocidade"
	 */
	public void setVelocidade(float velocidade) {
		this.velocidade = velocidade;
	}

	/**
	 * Setter of the property <tt>x</tt>
	 * @param x  The x to set.
	 * @uml.property  name="x"
	 */
	public void setX(int x) {
		this.x = x;
	}
	
	/**
	 * Setter of the property <tt>y</tt>
	 * @param y  The y to set.
	 * @uml.property  name="y"
	 */
	public void setY(int y) {
		this.y = y;
	}

	public float getTempoTotalMovimentacao() {
		/*float tempoTotal = 0;
		if((this.percurso!=null)&&(this.percurso.npoints>0))
		{
			for(int i=1; i<this.percurso.npoints; i++)
			{
				tempoTotal+=Point.distance(this.percurso.xpoints[i-1], this.percurso.ypoints[i-1], this.percurso.xpoints[i], this.percurso.ypoints[i])/this.getVelocidadeInstantanea(i-1);
			}
		}
		return tempoTotal;*/
		return 10;
	}

	@Override
	public int getHeight() {
		return iHeight;
	}

	@Override
	public int getWidth() {
		return iWidth;
	}

	@Override
	public void setHeight(int height) {
		iHeight = height;
	}

	@Override
	public void setWidth(int width) {
		iWidth = width;
	}
	
	public SentidoVehicle getSentido() {
		return svSentido;
	}

	public void setSentido(SentidoVehicle direction) {
		svSentido = direction;
	}
	
	@Override
	public void move()
	{
		if (currentPoint == -1)
		{
			// If current position isn't defined we take the start position
			xCurrent = x;
			yCurrent = y;
			isMoving = true;
			
			currentPoint = 0;
			
			// Calculate the current direction according to the next point way
			decideDirection();
		}
		
		if (isMoving)
			movePos();
		
		notifyObservers(obj2DOld);
	}
	
	@Override
	public void moveInitialPos()
	{
		currentPoint = -1;
	}

	private void decideDirection()
	{
		if (currentPoint < pontos.size())
		{
			Point currPoint = pontos.get(currentPoint);
			if (xCurrent == currPoint.x)
			{
				svSentido = SentidoVehicle.VERTICAL;
				if (yCurrent < currPoint.y)
					isPosIncreasing = true;
				else
					isPosIncreasing = false;
			}
			else
			{
				svSentido = SentidoVehicle.HORIZONTAL;
				if (xCurrent < currPoint.x)
					isPosIncreasing = true;
				else
					isPosIncreasing = false;
			}
		}
		else
			isMoving = false;
	}
	
	private void movePos()
	{
		Point currPoint = pontos.get(currentPoint);
		
		if (svSentido == SentidoVehicle.VERTICAL)
		{
			if (isPosIncreasing)
			{
				yCurrent += velocidade;
				// After calculating the new position, if it has reached the current point of reference
				if (yCurrent >= currPoint.y)
				{
					yCurrent = currPoint.y;
					currentPoint++;
					decideDirection();
				}
			}
			else
			{
				yCurrent -= velocidade;
				if (yCurrent <= currPoint.y)
				{
					yCurrent = currPoint.y;
					currentPoint++;
					decideDirection();
				}
			}
		}
		else
		{
			if (isPosIncreasing)
			{
				xCurrent += velocidade;
				if (xCurrent >= currPoint.x)
				{
					xCurrent = currPoint.x;
					currentPoint++;
					decideDirection();
				}
			}
			else
			{
				xCurrent -= velocidade;
				if (xCurrent <= currPoint.x)
				{
					xCurrent = currPoint.x;
					currentPoint++;
					decideDirection();
				}
			}
		}
	}
}