/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.execucao;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileReader;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import br.edu.fei.dcc.raciocinioespacial.simulador.persistencia.PersistenciaSimulacao;
import br.edu.fei.dcc.raciocinioespacial.simulador.simulacao.EstadoSimulacao;
import br.edu.fei.dcc.raciocinioespacial.simulador.simulacao.Simulacao;

/**
 * @author Thiago
 *
 */
public class Simulador extends JFrame implements Observer {

	private class BtnPlayListener implements ActionListener
	{

		@Override
		public void actionPerformed(ActionEvent e) {
			Simulador.this.getSimulacao().setContextSimulacao(Simulador.this.getEspaco().getGraphics());
			if(Simulador.this.getSimulacao().getEstado()==EstadoSimulacao.AGUARDANDO)
			{
				Simulador.this.getSimulacao().setDlgDetails(Simulador.this.dlgDetails);
				Simulador.this.getSimulacao().setState(EstadoSimulacao.EM_EXECUCAO);
				Simulador.this.getSimulacao().startSim();
			}
			else
			{
				Simulador.this.getSimulacao().setState(EstadoSimulacao.AGUARDANDO);
				Simulador.this.getSimulacao().stopSim();
			}
			Simulador.this.setBtnPlayImageButton();
		}
	}
	
	private class ChkPerfilListener implements ActionListener
	{

		@Override
		public void actionPerformed(ActionEvent e) {
			Simulador.this.getSimulacao().setIsShowProfile(Simulador.this.getChkPerfil().getSelectedObjects()!=null);
			Simulador.this.mostrarPerfilProfundidade();
		}
	}
	
	private class ChkDetailsListener implements ActionListener
	{

		@Override
		public void actionPerformed(ActionEvent e) {
			Simulador.this.mostrarDetails();
		}
	}
	
	private class DepthProfileWindowListener implements WindowListener
	{

		@Override
		public void windowActivated(WindowEvent e) {}

		@Override
		public void windowClosed(WindowEvent arg0) {}

		@Override
		public void windowClosing(WindowEvent arg0) {
			if(Simulador.this.getChkPerfil()!=null)
			{
				Simulador.this.getChkPerfil().setSelected(false);
			}
		}

		@Override
		public void windowDeactivated(WindowEvent arg0) {}

		@Override
		public void windowDeiconified(WindowEvent arg0) {}

		@Override
		public void windowIconified(WindowEvent arg0) {}

		@Override
		public void windowOpened(WindowEvent arg0) {}
		
	}
	
	private class DetailsWindowListener implements WindowListener
	{
		@Override
		public void windowActivated(WindowEvent e) {}

		@Override
		public void windowClosed(WindowEvent arg0) {}

		@Override
		public void windowClosing(WindowEvent arg0) {
			if(Simulador.this.getChkDetails()!=null)
			{
				Simulador.this.getChkDetails().setSelected(false);
			}
		}

		@Override
		public void windowDeactivated(WindowEvent arg0) {}

		@Override
		public void windowDeiconified(WindowEvent arg0) {}

		@Override
		public void windowIconified(WindowEvent arg0) {}

		@Override
		public void windowOpened(WindowEvent arg0) {}
	}

	private class SimuladorWindowListener implements WindowListener
	{

		@Override
		public void windowActivated(WindowEvent e) {}

		@Override
		public void windowClosed(WindowEvent arg0) {}

		@Override
		public void windowClosing(WindowEvent arg0) {
			Simulador.this.getDlgPerfil().dispose();
		}

		@Override
		public void windowDeactivated(WindowEvent arg0) {}

		@Override
		public void windowDeiconified(WindowEvent arg0) {}

		@Override
		public void windowIconified(WindowEvent arg0) {}

		@Override
		public void windowOpened(WindowEvent arg0) {}
		
	}

	private static final long serialVersionUID = 1L;
	
	private static final Dimension tamanhoPadrao = new Dimension(1000,700);
		
	public static void main(String[] args) {
		try{
			File file = new File("Properties\\filespath.properties");
	        FileReader fileR = new FileReader(file);
	        Properties prop = new Properties();
	        prop.load(fileR);
	        String sNameFile = prop.getProperty("mainTeste");
	        
			Simulacao sim = new PersistenciaSimulacao(sNameFile).obter();
			Simulador simulador = new Simulador(sim);
			simulador.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			simulador.setVisible(true);

		}catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	public static String REAL_PATH = ".";
	
	private JButton btnPlay;
	private JCheckBox chkPerfil;
	private JCheckBox chkDetails;
	private JLabel lblEstado;
	private JPanel conteudo;
	private JPanel espaco;
	private JPanel details;
	private JPanel menu;
	private DlgDepthProfile dlgProfile;
	private DlgDetails      dlgDetails;
	private Simulacao simulacao;
	
	private Dimension sizeWindow;
	
	public Simulador()
	{
		this(new Simulacao(false, true, null), Simulador.tamanhoPadrao);		
	}
	
	public Simulador(Simulacao simulacao)
	{
		this(simulacao, Simulador.tamanhoPadrao);
	}
	
	public Simulador(Simulacao simulacao, String realPath)
	{
		this(simulacao, Simulador.tamanhoPadrao);
		REAL_PATH = realPath;
	}
	
	public Simulador(Simulacao simulacao, Dimension tamanhoTela)
			throws HeadlessException {
		super();
		this.simulacao = simulacao;
		this.simulacao.addObserver(this);
		this.sizeWindow = tamanhoTela;
		this.init();
	}
	
	private JLabel getLblEstado()
	{
		if(this.lblEstado == null)
		{
			this.lblEstado = new JLabel();
			this.lblEstado.setForeground(Color.WHITE);
			this.lblEstado.setFont(new Font("Arial", Font.PLAIN, 10));
			this.lblEstado.setText("Waiting to start simulation...");
			this.lblEstado.setAlignmentX(Component.LEFT_ALIGNMENT);
			this.lblEstado.setSize(new Dimension(180, 16));
		}
		return this.lblEstado;
	}
	
	private JButton getBtnPlay() {
		if(this.btnPlay == null)
		{
			this.btnPlay = new JButton();
			this.setBtnPlayImageButton();
			this.btnPlay.setToolTipText(Messages.getString("Simulador.Iniciar_Execucao")); //$NON-NLS-1$
			this.btnPlay.setBackground(Color.GRAY);
			this.btnPlay.setBorderPainted(false);
			this.btnPlay.addActionListener(new BtnPlayListener());
		}
		return this.btnPlay;
	}

	JCheckBox getChkPerfil() {
		if(this.chkPerfil==null)
		{
			this.chkPerfil = new JCheckBox();
			this.chkPerfil.setBackground(Color.GRAY);
			this.chkPerfil.setText(Messages.getString("Simulador.Mostrar_PerfilProfundidade")); //$NON-NLS-1$
			this.chkPerfil.setSelected(this.getSimulacao().getMostrarPerfil());
			this.chkPerfil.setForeground(Color.WHITE);
			this.chkPerfil.addActionListener(new ChkPerfilListener());
		}
		return this.chkPerfil;
	}
	
	JCheckBox getChkDetails() {
		if(this.chkDetails==null)
		{
			this.chkDetails = new JCheckBox();
			this.chkDetails.setBackground(Color.GRAY);
			this.chkDetails.setText(Messages.getString("Simulador.Mostrar_DetalhesSim")); //$NON-NLS-1$
			this.chkDetails.setSelected(this.getSimulacao().getMostrarDetalhes());
			this.chkDetails.setForeground(Color.WHITE);
			this.chkDetails.addActionListener(new ChkDetailsListener());
		}
		return this.chkDetails;
	}

	public JPanel getConteudo() {
		if (this.conteudo == null) {
			this.conteudo = new JPanel();
			this.conteudo.setLayout(new BorderLayout());
			this.conteudo.add(this.getMenu(), BorderLayout.NORTH);
			this.conteudo.add(this.getEspaco(), BorderLayout.CENTER);
		}
		return this.conteudo;
	}

	public JPanel getEspaco() {
		if (this.espaco == null) {
			this.espaco = new JPanel();
			this.espaco.setLayout(new GridBagLayout());
			this.espaco.setBackground(Color.WHITE);
			this.espaco.setSize(new Dimension((this.sizeWindow.height - 30),this.sizeWindow.width));
		}
		return this.espaco;
	}
	
	public JPanel getDetails() {
		if (this.details == null) {
			this.details = new JPanel();
			this.details.setLayout(new GridBagLayout());
			this.details.setBackground(Color.GREEN);
			this.details.setSize(new Dimension((int)(this.sizeWindow.height*.8),150));
		}
		return this.details; 
	}

	public JPanel getMenu() {
		if (this.menu == null) {
			this.menu = new JPanel();
			this.menu.setLayout(new GridLayout(1,3));
			this.menu.setBackground(Color.GRAY);
			this.menu.setSize(new Dimension(30,this.sizeWindow.width));
			this.menu.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1));
			
			JPanel jpnBtnPlay = new JPanel();
			jpnBtnPlay.setLayout(new BorderLayout());
			jpnBtnPlay.setBackground(Color.GRAY);
			this.menu.add(jpnBtnPlay);
			jpnBtnPlay.add(this.getBtnPlay(), BorderLayout.WEST);
			
			JPanel jpnChecks = new JPanel();
			jpnChecks.setBackground(Color.GRAY);
			this.menu.add(jpnChecks);
			jpnChecks.add(this.getChkPerfil());
			jpnChecks.add(this.getChkDetails());
			
			JPanel jpnLblStatus = new JPanel();
			jpnLblStatus.setLayout(new BorderLayout());
			jpnLblStatus.setBackground(Color.GRAY);
			this.menu.add(jpnLblStatus);
			jpnLblStatus.add(this.getLblEstado(), BorderLayout.EAST);
		}
		return this.menu;
	}

	public DlgDepthProfile getDlgPerfil() {
		if(this.dlgProfile==null)
		{
			this.dlgProfile = new DlgDepthProfile(this, simulacao, new Dimension(400, 300));
			this.dlgProfile.setLocation(
					this.getWindowSize().width - (this.getWindowSize().width/2), 
					this.getWindowSize().height - 280);
			this.dlgProfile.addWindowListener(new DepthProfileWindowListener());
			this.dlgProfile.setResizable(false);
		}
		return this.dlgProfile;
	}
	
	public DlgDetails getDlgDetails() {
		if(this.dlgDetails==null)
		{
			this.dlgDetails = new DlgDetails(this, simulacao, new Dimension(453, 100));
			this.dlgDetails.setLocation(0, this.getWindowSize().height - 80);
			this.dlgDetails.addWindowListener(new DetailsWindowListener());
			this.dlgDetails.setResizable(false);
		}
		return this.dlgDetails;
	}

	public Simulacao getSimulacao() {
		return simulacao;
	}

	public Dimension getWindowSize() {
		return sizeWindow;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	public void init() {
		this.setSize(this.getWindowSize());
		this.setContentPane(this.getConteudo());
		this.getSimulacao().setContextSimulacao(this.getEspaco().getGraphics());
		this.setTitle(Messages.getString("Simulador.Titulo") + this.getSimulacao().getNome()); //$NON-NLS-1$
		this.addWindowListener(new SimuladorWindowListener());
		this.getSimulacao().drawInitialState();
		this.mostrarPerfilProfundidade();
	}
	
	void mostrarPerfilProfundidade()
	{
		this.getDlgPerfil().setVisible(this.getSimulacao().getMostrarPerfil());
	}
	
	void mostrarDetails()
	{
		this.getDlgDetails().setVisible(chkDetails.isSelected());
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
		setBtnPlayImageButton();
	}
	
	void setBtnPlayImageButton()
	{
		if(this.getSimulacao().getEstado()==EstadoSimulacao.AGUARDANDO)
		{
			this.btnPlay.setIcon(new ImageIcon(this.getClass().getResource(Messages.getString("Simulador.Iniciar")))); //$NON-NLS-1$
			this.btnPlay.setToolTipText(Messages.getString("Simulador.Iniciar_Execucao")); //$NON-NLS-1$
		}
		else
		{
			this.btnPlay.setIcon(new ImageIcon(this.getClass().getResource(Messages.getString("Simulador.Parar")))); //$NON-NLS-1$
			this.btnPlay.setToolTipText(Messages.getString("Simulador.Parar_Execucao")); //$NON-NLS-1$
		}
	}

	public void setPerfil(DlgDepthProfile perfil) {
		this.dlgProfile = perfil;
	}

	public void setSimulacao(Simulacao simulacao) {
		this.simulacao = simulacao;
	}
	
	public void setWindowSize(Dimension tamanhoTela) {
		this.sizeWindow = tamanhoTela;
	}

	public void setLblEstado(String s)
	{
		this.getLblEstado().setText(s);
	}
	
	public void update(Observable o, Object arg) {
		this.setBtnPlayImageButton();
		if(arg instanceof String)
			this.setLblEstado((String)arg);
	}
}