/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.movimentacao;

import java.io.File;
import java.io.FileWriter;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.GregorianCalendar;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.IEntidadeMovel;
import br.edu.fei.dcc.raciocinioespacial.simulador.execucao.Simulador;
import br.edu.fei.dcc.raciocinioespacial.simulador.perfilprofundidade.PerfilProfundidade;
import br.edu.fei.dcc.raciocinioespacial.simulador.persistencia.SaidaPerfilProfundidade;
import br.edu.fei.dcc.raciocinioespacial.simulador.prologCommunication.Translator;
import br.edu.fei.dcc.raciocinioespacial.simulador.simulacao.Simulacao;
import br.edu.fei.dcc.raciocinioespacial.simulador.util.Utils;

public class GerenciadorMovimentacao implements Runnable {
	private Simulacao simulacao;
	private float fInterval;
	private boolean isToStop;
	private Thread thread;
	
	private final int TAXA_ASSERTS = 5; // TODO Resgatar de mais um atributo da sim
	
	/** It starts the thread which refreshes the movements constantly */
	public void startMovements()
	{
		this.thread = new Thread(this);
		this.thread.setPriority(Thread.MIN_PRIORITY);
		
		this.thread.start();
	}
	
	public void stopMovements()
	{
		this.isToStop = true;
	}
	
	public GerenciadorMovimentacao(Simulacao simulacao, float intervalo, ArrayList<IEntidadeMovel> entidades) {
		super();
		this.simulacao = simulacao;
		this.fInterval = intervalo;
	}

	/**
	 * It run the new thread
	 */
	public void run() 
	{
		try
		{
			this.simulacao.notifyObservers("Opening prolog communication...");
			Translator handlerProlog = new Translator(Simulador.REAL_PATH);

			if (handlerProlog.startPrologConnection())
			{
				this.simulacao.notifyObservers("Simulation running..."); //$NON-NLS-1$
				long timeToWait = 0;
				this.isToStop = false;

				int iAssertsCount = TAXA_ASSERTS;

				simulacao.getProfile().resetCountCampo();

				long timeStartRun = GregorianCalendar.getInstance().getTimeInMillis();

				while(simulacao.getAgent().hasMovement())
				{
					long timeStartProc = GregorianCalendar.getInstance().getTimeInMillis();
					timeToWait += this.fInterval;

					// At the beginning of each movement cycle we need to update the draw of the streets
					simulacao.updateStreets();
					simulacao.updateDados((timeStartProc-timeStartRun), simulacao.getProfile().getCountCampo());

					simulacao.getAgent().move();

					for(IEntidadeMovel iem : simulacao.getObjetos())
						iem.move();

					if (isToStop)
						break;

					if (iAssertsCount >= TAXA_ASSERTS)
					{
						long relatedTime = simulacao.getProfile().getCountCampo();
						simulacao.getProfile().calcCampoVisaoInstantaneo();
						iAssertsCount = 0;
						if (relatedTime > 170 && relatedTime < 185)
							iAssertsCount = 0;
						
						handlerProlog.assertDepthProfile(simulacao.getProfile().getCurrentProfile());
						//System.out.println(simulacao.getProfile().getCurrentProfile());
						simulacao.setBoolLights(handlerProlog.queryObjectInColisionRoute(relatedTime),
								handlerProlog.queryVelocityReduceNeeded(relatedTime),
								handlerProlog.queryUnsafeOvertaking(relatedTime));
							
					}
					iAssertsCount++;

					timeToWait-=(GregorianCalendar.getInstance().getTimeInMillis()-timeStartProc);
					if(timeToWait>0)
					{
						Utils.aguardar(timeToWait);
						timeToWait = 0;
					}
				}
				
				simulacao.endSimulation(!isToStop);
				simulacao.getAgent().moveInitialPos();
				for(IEntidadeMovel iem : simulacao.getObjetos())
				{
					iem.moveInitialPos();
				}
				// Save all profiles in file
				// TODO permitir configurar onde salvar o perfil
				new SaidaPerfilProfundidade(simulacao.getProfile().getAllProfile(), simulacao.getNome(), PerfilProfundidade.EXTENSAO).salvarPerfilProfundidade();
			}
			else
				this.simulacao.notifyObservers("ERROR - Prolog communication could not be established! ");
			
			handlerProlog.endPrologConnection();
		}
		catch(Exception ex)
		{
			this.simulacao.notifyObservers("An unexpected error occurred ");
			OutputStreamWriter out = null;
			
			try
			{
				File f = new File("C:\\Temp\\"+
						String.format("%s.log", "LOG-STV-"+GregorianCalendar.getInstance().getTime()));
				if(f.exists())
					f.delete();
				out = new FileWriter(f);
				out.write(ex.toString()+ex.getMessage());
				out.flush();
				out.close();
			}catch(Exception ex2)
			{}
			simulacao.getContextSimulation().drawString(ex.toString()+"\n"+ex.getMessage(), 10, 10);
		}
	}
}