package br.edu.fei.dcc.raciocinioespacial.simulador.prologCommunication;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.Properties;

/**
* 
* @author João
*/
public class Translator extends Thread
{
    private static int ASSERT_DEPTH_PROFILE = 0;
    private static int QUERY_OBJECT_IN_COLISION_ROUTE = 1;
    private static int QUERY_UNSAFE_OVERTAKING = 2;
    private static int QUERY_VELOCITY_REDUCE_NEEDED = 3;
    private static int END_PROLOG = 4;
    boolean isRunning;
    
    //Will read the lines writen by prolog program
    BufferedReader prologOutput ; 
    //Will write lines to be read by the prolog program
    BufferedWriter prologInput ;
    //This process will start and keep running the XXX file with SWI-Prolog
    private Process prologProcess;
    //This socket will comunicate with ConnectorServer, as the java endpoint
    private Socket socket;
    //This long value will be a counter for temptatives of communication with prolog
    private long translatorCounter;
    //private PrologResponseHolder responseHolder = new PrologResponseHolder();
    private PrologResponseHolder responseHolder = PrologResponseHolder.getInstance();
    
    Connector connectorServer;
	private String sPath;
    
    public Translator(String sRealPath)
    {
    	sPath = sRealPath;
    }
    
	//This method retuns the number of request made to the prolog, and also iterates it
    private long getCounter()
    {   
        return (++this.translatorCounter);
    }
    
	//The run() method of this thread... it will keep waiting responses from prolog, and when it receives a response, updates the responseHolder object
    public void run()
    { 

        {
            String output;

            while(this.isRunning)
            {
                //when there is a response, wake-ups every method that is waiting the response holder object
                try
                {
                    output = prologOutput.readLine();
                    this.responseHolder.setPrologCounter(Long.parseLong(output.split(",")[0]));
                    this.responseHolder.setPrologResponse(output.split(",")[1].trim());
                    synchronized(this.responseHolder)
                    {
                        if(this.responseHolder.getPrologResponse().equals("end_prolog"))
                        {
                            this.isRunning=false;
                            //System.out.println("Translator gone");
                        }
                        //System.out.println("waking");
                        this.responseHolder.notify();
                    }
                    
                    
                }
                catch(Exception e)
                {
                    System.out.println(e);
                }
            }   
        }
    }
    
	//This method will start a prolog process, make connection to is and also start the tread from this object
    public boolean startPrologConnection() 
            throws Exception
    {    
        //Starts a socket server waiting for a connection
        connectorServer = new Connector(22105);
        connectorServer.start();
        isRunning=true;
        try
        {
            //Try a connection with connectorServer
            socket = new Socket("127.0.0.1", 22105);
            socket.setSoLinger(true, 0);
            //Makes prologOutput to receive any message broadcasted from socket input stream
            prologOutput = new BufferedReader(new InputStreamReader(socket.getInputStream())) ;
            //Makes prologInput send any message via the socket output stream
            prologInput = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())) ; 
            
        }
        catch (Exception e)
        {
            System.out.println(e.getMessage());
        }

        this.start();
        
        File file = new File(sPath+"\\Properties\\filespath.properties");
        FileReader fileR = new FileReader(file);
        Properties prop = new Properties();
        prop.load(fileR);
        //System.out.println(file.getAbsoluteFile());
        String prologExecPath = prop.getProperty("prologexecpath");
        file = new File(sPath+"\\PrologFile\\spatialReasoning.pl");
        String prologFilePath = file.getAbsolutePath();
        System.out.println(prologFilePath);
        //Try to start the XXX file in the specified path via SWI-Prolog
        try 
        {
            this.prologProcess = Runtime.getRuntime().exec("\""+prologExecPath + "\" -f \"" + prologFilePath+"\"") ; 
        } 
        catch(Exception e) 
        {
            System.out.println(e);
            return false;
        }
        Thread.sleep(1000);
        return true;
    }
    
	//This method will end the prolog process and also every running thread related to it
    public synchronized boolean endPrologConnection() 
            throws Exception
    {
        long prologRequestNumber = this.getCounter();
        PrologResponseHolder holder = PrologResponseHolder.getInstance();
        //boolean response;
        
        synchronized(this.responseHolder)
        {
            this.prologInput.write("("+END_PROLOG+","+prologRequestNumber+","+"'end_prolog'"+").\n");
            this.prologInput.flush();
            while(holder.getPrologCounter() != prologRequestNumber)
            {

                try
                {   
                    this.responseHolder.wait();
                }
                catch(InterruptedException e)
                {System.out.println(e);}  
            }
        }
        
        this.connectorServer.isRunning = false;
        this.socket.close();
        this.socket = new Socket("127.0.0.1", 22105);
        this.socket.setSoLinger(true, 0);
        this.prologProcess.destroy();
        //this.connectorServer=null;
        //this.prologInput.close();
        //this.prologOutput.close();
        Thread.sleep(2000);
        return (true);
        
    }
    
    //This method will  assert a peak in prolog knowledge base... 
    public synchronized boolean assertDepthProfile(String depthProfile) throws Exception
    {
        long prologRequestNumber = this.getCounter(); 
        PrologResponseHolder holder = PrologResponseHolder.getInstance();
        
        boolean response = false;
        
        synchronized(this.responseHolder)
        {
            this.prologInput.write("("+ASSERT_DEPTH_PROFILE+","+prologRequestNumber+","+depthProfile+").\n");
            this.prologInput.flush();
            while(holder.getPrologCounter() != prologRequestNumber || holder.getPrologResponse().equals("end_prolog"))
            {

                try
                {   
                    this.responseHolder.wait();
                }
                catch(InterruptedException e)
                {System.out.println(e);}  
            }
            if(!holder.getPrologResponse().equals("end_prolog"))
            {
                response = Boolean.parseBoolean(holder.getPrologResponse());
            }
        }
        return (response);
    }
    
	//This method will  ast to prolog if there is an object is in collision route.. 
    public synchronized boolean queryObjectInColisionRoute(long relatedTime) throws Exception
    {
        long prologRequestNumber = this.getCounter(); 
        PrologResponseHolder holder = PrologResponseHolder.getInstance();
        
        boolean response = false;
        
        synchronized(this.responseHolder)
        {
            this.prologInput.write("("+QUERY_OBJECT_IN_COLISION_ROUTE+","+prologRequestNumber+","+relatedTime+").\n");
            this.prologInput.flush();
            
            while(holder.getPrologCounter() != prologRequestNumber || holder.getPrologResponse().equals("end_prolog"))
            {

                try
                {   
                    this.responseHolder.wait();
                }
                catch(InterruptedException e)
                {System.out.println(e);}  
            }
            if(!holder.getPrologResponse().equals("end_prolog"))
            {
                response = Boolean.parseBoolean(holder.getPrologResponse());
            }
        }
        return (response);
        
    }
    
	//This method will  ast to prolog if it is safe to overtake at the given moment...
    public synchronized boolean queryUnsafeOvertaking(long relatedTime) throws Exception
    {
        long prologRequestNumber = this.getCounter(); 
        PrologResponseHolder holder = PrologResponseHolder.getInstance();
        
        boolean response = false;
        
        synchronized(this.responseHolder)
        {
            this.prologInput.write("("+QUERY_UNSAFE_OVERTAKING+","+prologRequestNumber+","+relatedTime+").\n");
            this.prologInput.flush();
            
            while(holder.getPrologCounter() != prologRequestNumber || holder.getPrologResponse().equals("end_prolog"))
            {

                try
                {   
                    this.responseHolder.wait();
                }
                catch(InterruptedException e)
                {System.out.println(e);}  
            }
            if(!holder.getPrologResponse().equals("end_prolog"))
            {
                response = Boolean.parseBoolean(holder.getPrologResponse());
            }
        }
        return (response);
    }
    
	//This method will  ast to prolog if it is necessary to reduce the velocity to avoid future collisions...
    public synchronized boolean queryVelocityReduceNeeded(long relatedTime) throws Exception
    {
        long prologRequestNumber = this.getCounter(); 
        PrologResponseHolder holder = PrologResponseHolder.getInstance();
        
        boolean response = false;
        
        synchronized(this.responseHolder)
        {
            this.prologInput.write("("+QUERY_VELOCITY_REDUCE_NEEDED+","+prologRequestNumber+","+relatedTime+").\n");
            this.prologInput.flush();
            
            while(holder.getPrologCounter() != prologRequestNumber || holder.getPrologResponse().equals("end_prolog"))
            {

                try
                {   
                    this.responseHolder.wait();
                }
                catch(InterruptedException e)
                {System.out.println(e);}  
            }
            if(!holder.getPrologResponse().equals("end_prolog"))
            {
                response = Boolean.parseBoolean(holder.getPrologResponse());
            }
        }
        return (response);
    }
    
}
