/**
 * 
 */
package br.edu.fei.dcc.raciocinioespacial.simulador.simulacao;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.geom.RectangularShape;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;

import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Agent2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.IEntidadeMovel;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Pedestrian2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Street2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.entidades.Vehicle2D;
import br.edu.fei.dcc.raciocinioespacial.simulador.execucao.DlgDetails;
import br.edu.fei.dcc.raciocinioespacial.simulador.movimentacao.GerenciadorMovimentacao;
import br.edu.fei.dcc.raciocinioespacial.simulador.perfilprofundidade.PerfilProfundidade;
import br.edu.fei.dcc.raciocinioespacial.simulador.util.Utils;

public class Simulacao extends Observable implements ISimulacao, Observer, Serializable {

	private static final long serialVersionUID = 1L;
	private ArrayList<IEntidadeMovel> iemObjects;
	private ArrayList<Street2D> ieStreets;
	private Agent2D iemAgent;
	private boolean isShowProfile;
	private boolean isShowDetails;
	private boolean isPaintRoad;
	private Graphics contextoSimulacao;
	private Graphics contextoPerfil;
	private JLabel jLblTime;
	private JLabel jLblCountProfiles;
	private DlgDetails dlgDetails;
	private String sName;
	private float fInterval;
	private EstadoSimulacao esState;
	private GerenciadorMovimentacao gerMovimentacao;
	private PerfilProfundidade profile;
	private static Rectangle limitProfile = new Rectangle(10,50,375,225);
	private static Rectangle limitGraphicProfile = new Rectangle(25,55,360,210);
	
	public Simulacao(String nome, float intervalo, boolean isShowProfile,
			boolean aIsShowDetails, boolean isPaintRoad, Graphics contexto)
	{
		super();
		this.sName = nome;
		this.fInterval = intervalo;
		this.isShowProfile = isShowProfile;
		this.isShowDetails = aIsShowDetails;
		this.isPaintRoad = isPaintRoad;
		this.contextoSimulacao = contexto;
		this.iemObjects = new ArrayList<IEntidadeMovel>();
		this.ieStreets  = new ArrayList<Street2D>();
		this.esState = EstadoSimulacao.AGUARDANDO;
	}
	
	public Simulacao(String nome, float intervalo, boolean isShowProfile,
			boolean aIsShowCourse, boolean isPaintRoad){
		this(nome, intervalo, isShowProfile, aIsShowCourse, isPaintRoad, null);
	}
	
	public Simulacao(boolean isShowProfile, boolean aIsShowCourse,
			Graphics contexto) {
		this("", 100, isShowProfile, aIsShowCourse, false, contexto);
	}
	
	private void addObservable(IEntidadeMovel entidade, ArrayList<IEntidadeMovel> colecao)
	{
		if(entidade instanceof Pedestrian2D)
		{
			// If the entity is a Pedestrian2D or its subclasses
			// Take it in a collection for movement treatments  
			Pedestrian2D obj = (Pedestrian2D)entidade;
			obj.addObserver(this);
			colecao.add(obj);
		}
	}
	
	public Graphics getContextSimulation() {
		return contextoSimulacao;
	}
	
	public EstadoSimulacao getEstado() {
		return esState;
	}
	
	public float getIntervalo() {
		return fInterval;
	}
	public boolean getMostrarDetalhes() {
		return isShowDetails;
	}
	public boolean getIsPaintRoads() {
		return isPaintRoad;
	}
	public boolean getMostrarPerfil() {
		return isShowProfile;
	}
	public String getNome() {
		return sName;
	}
	public ArrayList<IEntidadeMovel> getObjetos() {
		return iemObjects;
	}
	public ArrayList<Street2D> getStreetsArray() {
		return ieStreets;
	}
	public Agent2D getAgent() {
		return iemAgent;
	}
	
	/**
	 * It creates and starts the GerenciadorMovimentacao, which manages all objects movements 
	 */
	public void startMovement()
	{
		if(this.gerMovimentacao==null)
		{
			ArrayList<IEntidadeMovel> entidades = new ArrayList<IEntidadeMovel>();
			this.addObservable(this.getAgent(), entidades);
			for(IEntidadeMovel entidade : this.getObjetos())
			{
				this.addObservable(entidade, entidades);
			}
			this.gerMovimentacao = new GerenciadorMovimentacao(this, this.fInterval, entidades);
			this.profile = new PerfilProfundidade((Agent2D)this.iemAgent, Pedestrian2D.transformEntidadeMovelToObjeto2D(this.iemObjects),
					this.getContextProfile(), limitGraphicProfile, limitProfile);
		}
	}
	
	/**
	 * It starts the simulation
	 */
	public void startSim()
	{
		this.notifyObservers("Starting the simulation...");
		this.startMovement();
		this.limparContextosGraficos();
		this.gerMovimentacao.startMovements();
	}
	
	private void limparContextosGraficos()
	{
		if ((this.getContextSimulation() != null) && (this.getContextProfile() != null))
		{
			this.limparContextoGrafico(this.getContextSimulation());
			this.limparContextoGrafico(this.getContextProfile());
		}
	}
	
	private void limparContextoGrafico(Graphics g)
	{
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, Short.MAX_VALUE, Short.MAX_VALUE);
		((Graphics2D)g).setBackground(Color.WHITE);
	}
	
	public void stopSim()
	{
		if(this.gerMovimentacao!=null)
		{
			this.gerMovimentacao.stopMovements();
			this.notifyObservers("Simulation stopped by user ");
		}
	}
	
	private void drawEntity(IEntidadeMovel entity)
	{
		if(entity instanceof Agent2D)
		{
			this.drawAgent((Agent2D)entity);
		}
		else if(entity instanceof Vehicle2D)
		{
			this.drawVehicle((Vehicle2D)entity);
		} 
		else if(entity instanceof Pedestrian2D)
		{
			this.drawPedestrian((Pedestrian2D)entity);
		}
	}
	
	private void drawStreet(Street2D oStreet)
	{
		Stroke s = new BasicStroke();
		
		Utils.drawShape(oStreet.getRect2D(), !this.getIsPaintRoads(), this.getIsPaintRoads(),
				this.getContextSimulation(), Color.DARK_GRAY, s);
	}

	public void drawInitialState()
	{
		// First of all, draw the streets!
		for(Street2D street : this.getStreetsArray())
		{
			this.drawStreet(street);
		}
		
		this.drawEntity(this.getAgent());
		
		for(IEntidadeMovel em : this.getObjetos())
		{
			this.drawEntity(em);
		}
	}
	
	private void drawPedestrian(Pedestrian2D obj)
	{
		Stroke s = new BasicStroke();
		Utils.drawShape(obj.getObjeto2D(), true, this.getContextSimulation(), obj.getCor(), s);
	}
	
	private void drawVehicle(Vehicle2D obj)
	{
		Stroke s = new BasicStroke();
		Utils.drawShape(obj.getObjeto2D(), false, this.getContextSimulation(), obj.getCor(), s);
	}
	
	private void drawAgent(Agent2D obs)
	{
		Stroke skCampo = new BasicStroke(1,BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0, new float[] { 5, 5 }, 0);
		Stroke sk = new BasicStroke();
		Utils.drawShape(obs.getObjeto2D(), false, this.getContextSimulation(), obs.getCor(), sk);
		
		Utils.drawArc(obs.getCampoVisao2D_Old(), this.getContextSimulation(), Color.WHITE, skCampo);
		Utils.drawArc(obs.getCampoVisao2D(), this.getContextSimulation(), Color.BLACK, skCampo);
	}
	
	public void setContextSimulacao(Graphics contexto) {
		this.contextoSimulacao = contexto;
	}
	
	public void setState(EstadoSimulacao estado) {
		this.esState = estado;
	}
	
	public void setInterval(float intervalo) {
		this.fInterval = intervalo;
	}
	
	public void setIsShowDetails(boolean mostrarDetalhes) {
		this.isShowDetails = mostrarDetalhes;
	}
	
	public void setIsPaintRoads(boolean isPaint) {
		this.isPaintRoad = isPaint;
	}
	
	public void setIsShowProfile(boolean mostrarPerfil) {
		this.isShowProfile = mostrarPerfil;
	}
	
	public void setName(String name) {
		this.sName = name;
	}
	
	public void setObjects(ArrayList<IEntidadeMovel> objetos) {
		this.iemObjects = objetos;
	}
	
	public void setStreets(ArrayList<Street2D> streets) {
		this.ieStreets = streets;
	}

	public void setAgent(Agent2D agent) {
		this.iemAgent = agent;
	}
	
	public void updateStreets()
	{
		for(Street2D street : getStreetsArray())
			this.drawStreet(street);
	}

	/* (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	@Override
	public synchronized void update(Observable o, Object obj2DOld) {
		if(o instanceof IEntidadeMovel)
		{
			if (obj2DOld != null)
				Utils.drawShape((RectangularShape)obj2DOld, true, this.getContextSimulation(), Color.WHITE, new BasicStroke());
				
			this.drawEntity((IEntidadeMovel) o);
		}
	}
	
	public void endSimulation(boolean isEnd)
	{
		this.esState = EstadoSimulacao.AGUARDANDO;
		
		if (isEnd)
			this.notifyObservers("Simulation has reached the end!");
	}
	
	public PerfilProfundidade getProfile() {
		return profile;
	}

	public Graphics getContextProfile() {
		return contextoPerfil;
	}

	public void setContextProfile(Graphics contextProfile) {
		this.contextoPerfil = contextProfile;
	}
	
	public void setLblTime(JLabel lblTime) {
		this.jLblTime = lblTime;
	}
	public void setLblCountProfiles(JLabel lblProfiles) {
		this.jLblCountProfiles = lblProfiles;
	}
	
	public void setDlgDetails(DlgDetails aDlgDetails)
	{
		this.dlgDetails = aDlgDetails;
	}

	@Override
	public void notifyObservers(Object o) {
		this.setChanged();
		super.notifyObservers(o);
	}
	
	public void setBoolLights(boolean isColision, boolean isBrake, boolean isOvertake)
	{
		if (dlgDetails != null)
		{
			dlgDetails.setImgColision(isColision);
			dlgDetails.setImgBrakeOn(isBrake);
			dlgDetails.setImgOvertake(isOvertake);
		}
	}

	public void updateDados(long timeStartProc, int iCountProfiles) {
		if (this.jLblTime != null)
			this.jLblTime.setText("Duration (ms): "+timeStartProc);
		if (this.jLblCountProfiles != null)
			this.jLblCountProfiles.setText("Assimilated profiles: "+iCountProfiles);
	}
}